@@ -1,6 +1,6 @@
 "This module defines rules and algorithms for generating C++ code."
 
-# Copyright (C) 2009-2011 Anders Logg
+# Copyright (C) 2009-2013 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -19,9 +19,10 @@
 #
 # Modified by Kristian B. Oelgaard 2011
 # Modified by Marie E. Rognes 2010
+# Modified by Martin Alnaes 2013
 #
 # First added:  2009-12-16
-# Last changed: 2011-11-28
+# Last changed: 2013-01-25
 
 # Python modules
 import re, numpy, platform
@@ -29,10 +30,15 @@ import re, numpy, platform
 # FFC modules
 from ffc.log import debug, error
 
-# Formatting rules
+# Mapping of restrictions
+_choose_map = {None: "", "+": "_0", "-": "_1"}
+
+# FIXME: MSA: Using a dict to collect functions in a namespace is weird
+#             and makes the code harder to follow, change to a class
+#             with member functions instead!
 # FIXME: KBO: format is a builtin_function, i.e., we should use a different name.
+# Formatting rules
 format = {}
-choose_map = {None: "", "+": "0", "-": "1"}
 
 # Program flow
 format.update({
@@ -92,14 +98,18 @@ format.update({
     "cos":            lambda v: "std::cos(%s)" % str(v),
     "sin":            lambda v: "std::sin(%s)" % str(v),
     "tan":            lambda v: "std::tan(%s)" % str(v),
+    "cosh":           lambda v: "std::cosh(%s)" % str(v),
+    "sinh":           lambda v: "std::sinh(%s)" % str(v),
+    "tanh":           lambda v: "std::tanh(%s)" % str(v),
     "acos":           lambda v: "std::acos(%s)" % str(v),
     "asin":           lambda v: "std::asin(%s)" % str(v),
     "atan":           lambda v: "std::atan(%s)" % str(v),
-    "erf":            lambda v:  "erf(%s)" % str(v),
-    "bessel_i":       lambda v, n:  "boost::math::tr1::cyl_bessel_i(%s, %s)" % (str(n), str(v)),
-    "bessel_j":       lambda v, n:  "boost::math::tr1::cyl_bessel_j(%s, %s)" % (str(n), str(v)),
-    "bessel_k":       lambda v, n:  "boost::math::tr1::cyl_bessel_k(%s, %s)" % (str(n), str(v)),
-    "bessel_y":       lambda v, n:  "boost::math::tr1::cyl_neumann(%s, %s)" % (str(n), str(v)),
+    "atan_2":         lambda v1,v2: "std::atan2(%s,%s)" % (str(v1),str(v2)),
+    "erf":            lambda v: "erf(%s)" % str(v),
+    "bessel_i":       lambda v, n: "boost::math::cyl_bessel_i(%s, %s)" % (str(n), str(v)),
+    "bessel_j":       lambda v, n: "boost::math::cyl_bessel_j(%s, %s)" % (str(n), str(v)),
+    "bessel_k":       lambda v, n: "boost::math::cyl_bessel_k(%s, %s)" % (str(n), str(v)),
+    "bessel_y":       lambda v, n: "boost::math::cyl_neumann(%s, %s)" % (str(n), str(v)),
     "absolute value": lambda v: "std::abs(%s)" % str(v),
     "sqrt":           lambda v: "std::sqrt(%s)" % str(v),
     "addition":       lambda v: _add(v),
@@ -117,24 +127,26 @@ format.update({
 
 # Geometry related variable names (from code snippets).
 format.update({
-    "entity index":     "c.entity_indices",
-    "num entities":     "m.num_entities",
-    "cell":             lambda s: "ufc::%s" % s,
-    "J":                lambda i, j: "J_%d%d" % (i, j),
-    "inv(J)":           lambda i, j: "K_%d%d" % (i, j),
-    "det(J)":           lambda r=None: "detJ%s" % choose_map[r],
-    "cell volume":      lambda r=None: "volume%s" % choose_map[r],
-    "circumradius":     lambda r=None: "circumradius%s" % choose_map[r],
-    "facet area":       "facet_area",
-    "scale factor":     "det",
-    "transform":        lambda t, j, k, r: _transform(t, j, k, r),
-    "normal component": lambda r, j: "n%s%s" % (choose_map[r], j),
-    "x coordinate":     "X",
-    "y coordinate":     "Y",
-    "z coordinate":     "Z",
-    "ip coordinates":   lambda i, j: "X%d[%d]" % (i, j),
-    "affine map table": lambda i, j: "FEA%d_f%d" % (i, j),
-    "coordinates":      lambda r=None: "x%s" % choose_map[r]
+    "entity index":       "c.entity_indices",
+    "num entities":       "num_global_entities",
+    "cell":               lambda s: "ufc::%s" % s,
+    "J":                  lambda i, j, m, n: "J[%d]" % _flatten(i, j, m, n),
+    "inv(J)":             lambda i, j, m, n: "K[%d]" % _flatten(i, j, m, n),
+    "det(J)":             lambda r=None: "detJ%s" % _choose_map[r],
+    "cell volume":        lambda r=None: "volume%s" % _choose_map[r],
+    "circumradius":       lambda r=None: "circumradius%s" % _choose_map[r],
+    "facet area":         "facet_area",
+    "min facet edge length": lambda r: "min_facet_edge_length",
+    "max facet edge length": lambda r: "max_facet_edge_length",
+    "scale factor":       "det",
+    "transform":          lambda t, i, j, m, n, r: _transform(t, i, j, m, n, r),
+    "normal component":   lambda r, j: "n%s%s" % (_choose_map[r], j),
+    "x coordinate":       "X",
+    "y coordinate":       "Y",
+    "z coordinate":       "Z",
+    "ip coordinates":     lambda i, j: "X%d[%d]" % (i, j),
+    "affine map table":   lambda i, j: "FEA%d_f%d" % (i, j),
+    "vertex_coordinates": lambda r=None: "vertex_coordinates%s" % _choose_map[r]
 })
 
 # UFC function arguments and class members (names)
@@ -145,8 +157,9 @@ format.update({
     "argument basis num":         "i",
     "argument derivative order":  "n",
     "argument values":            "values",
-    "argument coordinates":       "coordinates",
-    "facet":                      lambda r: "facet%s" % choose_map[r],
+    "argument coordinates":       "dof_coordinates",
+    "facet":                      lambda r: "facet%s" % _choose_map[r],
+    "vertex":                     "vertex",
     "argument axis":              "i",
     "argument dimension":         "d",
     "argument entity":            "i",
@@ -179,8 +192,8 @@ format.update({
     "local dof":                "dof",
     "basisvalues":              "basisvalues",
     "coefficients":             lambda i: "coefficients%d" %(i),
-    "num derivatives":          "num_derivatives",
-    "derivative combinations":  "combinations",
+    "num derivatives":          lambda t_or_g :"num_derivatives" + t_or_g,
+    "derivative combinations":  lambda t_or_g :"combinations" + t_or_g,
     "transform matrix":         "transform",
     "transform Jinv":           "Jinv",
     "dmats":                    lambda i: "dmats%s" %(i),
@@ -191,8 +204,9 @@ format.update({
                                 % (i, format["argument basis num"], format["argument basis num"], j),
     "dereference pointer":      lambda n: "*%s" % n,
     "reference variable":       lambda n: "&%s" % n,
-    "call basis":               lambda i, s: "evaluate_basis(%s, %s, coordinates, c);" % (i, s),
-    "call basis_derivatives":   lambda i, s: "evaluate_basis_derivatives(%s, n, %s, coordinates, c);" % (i, s),
+    "call basis":               lambda i, s: "evaluate_basis(%s, %s, x, vertex_coordinates, cell_orientation);" % (i, s),
+    "call basis all":           "evaluate_basis_all(values, x, vertex_coordinates, cell_orientation);",
+    "call basis_derivatives":   lambda i, s: "evaluate_basis_derivatives(%s, n, %s, x, vertex_coordinates, cell_orientation);" % (i, s),
 
     # quadrature code generators
     "integration points": "ip",
@@ -209,7 +223,7 @@ format.update({
     "function value":     lambda i: "F%d" % i,
     "nonzero columns":    lambda i: "nzc%d" % i,
     "weight":             lambda i: "W%d" % (i),
-    "psi name":           lambda c, f, co, d: _generate_psi_name(c,f,co,d),
+    "psi name":           lambda c, et, e, co, d, a: _generate_psi_name(c, et, e, co, d, a),
     # both
     "free indices":       ["r","s","t","u"],
     "matrix index":       lambda i, j, range_j: _matrix_index(i, str(j), str(range_j))
@@ -228,30 +242,33 @@ format.update({
 
 # Code snippets
 from codesnippets import *
+
 format.update({
-    "cell coordinates":     cell_coordinates,
-    "jacobian":             lambda n, r="": jacobian[n] % {"restriction": r},
-    "inverse jacobian":     lambda n, r="": inverse_jacobian[n] % {"restriction": r},
-    "jacobian and inverse": lambda n, r=None: format["jacobian"](n, choose_map[r]) +\
-                            "\n" + format["inverse jacobian"](n, choose_map[r]),
-    "facet determinant":    lambda n, r=None: facet_determinant[n] % {"restriction": choose_map[r]},
-    "fiat coordinate map":  lambda n: fiat_coordinate_map[n],
-    "generate normal":      lambda d, i: _generate_normal(d, i),
-    "generate cell volume": lambda d, i: _generate_cell_volume(d, i),
-    "generate circumradius": lambda d, i: _generate_circumradius(d, i),
-    "generate facet area":  lambda d: facet_area[d],
+    "compute_jacobian":         lambda tdim, gdim, r=None: \
+                                compute_jacobian[tdim][gdim] % {"restriction": _choose_map[r]},
+    "compute_jacobian_inverse": lambda tdim, gdim, r=None: \
+                                compute_jacobian_inverse[tdim][gdim] % {"restriction": _choose_map[r]},
+    "orientation":              lambda tdim, gdim, r=None: orientation_snippet % {"restriction": _choose_map[r]} if tdim != gdim else "",
+    "facet determinant":        lambda tdim, gdim, r=None: facet_determinant[tdim][gdim] % {"restriction": _choose_map[r]},
+    "fiat coordinate map":      lambda cell, gdim: fiat_coordinate_map[cell][gdim],
+    "generate normal":          lambda tdim, gdim, i: _generate_normal(tdim, gdim, i),
+    "generate cell volume":     lambda tdim, gdim, i: _generate_cell_volume(tdim, gdim, i),
+    "generate circumradius":    lambda tdim, gdim, i: _generate_circumradius(tdim, gdim, i),
+    "generate facet area":      lambda tdim, gdim: facet_area[tdim][gdim],
+    "generate min facet edge length": lambda tdim, gdim, r=None: min_facet_edge_length[tdim][gdim] % {"restriction": _choose_map[r]},
+    "generate max facet edge length": lambda tdim, gdim, r=None: max_facet_edge_length[tdim][gdim] % {"restriction": _choose_map[r]},
     "generate ip coordinates":  lambda g, num_ip, name, ip, r=None: (ip_coordinates[g][0], ip_coordinates[g][1] % \
-                                {"restriction": choose_map[r], "ip": ip, "name": name, "num_ip": num_ip}),
-    "scale factor snippet": scale_factor,
-    "map onto physical":    map_onto_physical,
-    "combinations":         combinations_snippet,
-    "transform snippet":    transform_snippet,
-    "evaluate function":    evaluate_f,
-    "ufc comment":          comment_ufc,
-    "dolfin comment":       comment_dolfin,
-    "header_h":             header_h,
-    "header_c":             header_c,
-    "footer":               footer
+                                {"restriction": _choose_map[r], "ip": ip, "name": name, "num_ip": num_ip}),
+    "scale factor snippet":    scale_factor,
+    "map onto physical":       map_onto_physical,
+    "combinations":            combinations_snippet,
+    "transform snippet":       transform_snippet,
+    "evaluate function":       evaluate_f,
+    "ufc comment":             comment_ufc,
+    "dolfin comment":          comment_dolfin,
+    "header_h":                header_h,
+    "header_c":                header_c,
+    "footer":                  footer
 })
 
 # Class names
@@ -262,18 +279,22 @@ format.update({
     "classname dofmap":  lambda prefix, i: "%s_dofmap_%d" % (prefix.lower(), i),
 
     "classname cell_integral":  lambda prefix, form_id, sub_domain:\
-               "%s_cell_integral_%d_%d" % (prefix.lower(), form_id, sub_domain),
+               "%s_cell_integral_%d_%s" % (prefix.lower(), form_id, sub_domain),
 
     "classname exterior_facet_integral":  lambda prefix, form_id, sub_domain:\
-              "%s_exterior_facet_integral_%d_%d" % (prefix.lower(), form_id, sub_domain),
+              "%s_exterior_facet_integral_%d_%s" % (prefix.lower(), form_id, sub_domain),
 
     "classname interior_facet_integral":  lambda prefix, form_id, sub_domain:\
-              "%s_interior_facet_integral_%d_%d" % (prefix.lower(), form_id, sub_domain),
+              "%s_interior_facet_integral_%d_%s" % (prefix.lower(), form_id, sub_domain),
+
+    "classname point_integral":  lambda prefix, form_id, sub_domain:\
+              "%s_point_integral_%d_%s" % (prefix.lower(), form_id, sub_domain),
 
     "classname form": lambda prefix, i: "%s_form_%d" % (prefix.lower(), i)
 })
 
-# Helper functions for formatting.
+# Helper functions for formatting
+
 def _declaration(type, name, value=None):
     if value is None:
         return "%s %s;" % (type, name);
@@ -393,9 +414,9 @@ def _inner_product(v, w):
 
     return result
 
-def _transform(type, j, k, r):
-    map_name = {"J": "J", "JINV": "K"}[type] + choose_map[r]
-    return (map_name + "_%d%d") % (j, k)
+def _transform(type, i, j, m, n, r):
+    map_name = {"J": "J", "JINV": "K"}[type] + _choose_map[r]
+    return (map_name + "[%d]") % _flatten(i, j, m, n)
 
 # FIXME: Input to _generate_switch should be a list of tuples (i, case)
 def _generate_switch(variable, cases, default=None, numbers=None):
@@ -504,115 +525,104 @@ def _matrix_index(i, j, range_j):
         access = format["add"]([irj, j])
     return access
 
-def _generate_psi_name(counter, facet, component, derivatives):
+def _generate_psi_name(counter, entitytype, entity, component, derivatives, avg):
     """Generate a name for the psi table of the form:
-    FE#_f#_C#_D###, where '#' will be an integer value.
+    FE#_f#_v#_C#_D###_A#, where '#' will be an integer value.
 
     FE  - is a simple counter to distinguish the various bases, it will be
           assigned in an arbitrary fashion.
 
     f   - denotes facets if applicable, range(element.num_facets()).
 
-    C   - is the component number if any (this does not yet take into account
-          tensor valued functions)
+    v   - denotes vertices if applicable, range(num_vertices).
 
-    D   - is the number of derivatives in each spatial direction if any. If the
-          element is defined in 3D, then D012 means d^3(*)/dydz^2."""
+    C   - is the component number if any (flattened in the case of tensor valued functions)
+
+    D   - is the number of derivatives in each spatial direction if any.
+          If the element is defined in 3D, then D012 means d^3(*)/dydz^2.
+
+    A   - denotes averaged over cell (AC) or facet (AF)
+    """
 
     name = "FE%d" % counter
-    if not facet is None:
-        name += "_f%d" % facet
+
+    if entitytype == "facet":
+        name += "_f%d" % entity
+    elif entitytype == "vertex":
+        name += "_v%d" % entity
+
     if component != () and component != []:
         name += "_C%d" % component
+
     if any(derivatives):
-        name += "_D" + "".join([str(d) for d in derivatives])
+        name += "_D" + "".join(map(str,derivatives))
+
+    if avg == "cell":
+        name += "_AC"
+    elif avg == "facet":
+        name += "_AF"
 
     return name
 
-def _generate_jacobian(cell_dimension, integral_type):
-    "Generate code for computing jacobian"
-
-    # Choose space dimension
-    if cell_dimension == 1:
-        jacobian = jacobian_1D
-        facet_determinant = facet_determinant_1D
-    elif cell_dimension == 2:
-        jacobian = jacobian_2D
-        facet_determinant = facet_determinant_2D
-    else:
-        jacobian = jacobian_3D
-        facet_determinant = facet_determinant_3D
-
-    # Check if we need to compute more than one Jacobian
-    if integral_type == "cell":
-        code  = jacobian % {"restriction":  ""}
-        code += "\n\n"
-        code += scale_factor
-    elif integral_type == "exterior facet":
-        code  = jacobian % {"restriction":  ""}
-        code += "\n\n"
-        code += facet_determinant % {"restriction": "", "facet" : "facet"}
-    elif integral_type == "interior facet":
-        code  = jacobian % {"restriction": choose_map["+"]}
-        code += "\n\n"
-        code += jacobian % {"restriction": choose_map["-"]}
-        code += "\n\n"
-        code += facet_determinant % {"restriction": choose_map["+"], "facet": "facet0"}
-
-    return code
-
-def _generate_normal(geometric_dimension, domain_type, reference_normal=False):
+def _generate_normal(tdim, gdim, domain_type, reference_normal=False):
     "Generate code for computing normal"
 
     # Choose snippets
-    direction = normal_direction[geometric_dimension]
-    normal = facet_normal[geometric_dimension]
+    direction = normal_direction[tdim][gdim]
+
+    assert (facet_normal[tdim].has_key(gdim)),\
+        "Facet normal not yet implemented for this tdim/gdim combo"
+    normal = facet_normal[tdim][gdim]
 
     # Choose restrictions
     if domain_type == "exterior_facet":
         code = direction % {"restriction": "", "facet" : "facet"}
         code += normal % {"direction" : "", "restriction": ""}
     elif domain_type == "interior_facet":
-        code = direction % {"restriction": choose_map["+"], "facet": "facet0"}
-        code += normal % {"direction" : "", "restriction": choose_map["+"]}
-        code += normal % {"direction" : "!", "restriction": choose_map["-"]}
+        code = direction % {"restriction": _choose_map["+"], "facet": "facet_0"}
+        code += normal % {"direction" : "", "restriction": _choose_map["+"]}
+        code += normal % {"direction" : "!", "restriction": _choose_map["-"]}
     else:
         error("Unsupported domain_type: %s" % str(domain_type))
     return code
 
-def _generate_cell_volume(geometric_dimension, domain_type):
+def _generate_cell_volume(tdim, gdim, domain_type):
     "Generate code for computing cell volume."
 
     # Choose snippets
-    volume = cell_volume[geometric_dimension]
+    volume = cell_volume[tdim][gdim]
 
     # Choose restrictions
     if domain_type in ("cell", "exterior_facet"):
         code = volume % {"restriction": ""}
     elif domain_type == "interior_facet":
-        code = volume % {"restriction": choose_map["+"]}
-        code += volume % {"restriction": choose_map["-"]}
+        code = volume % {"restriction": _choose_map["+"]}
+        code += volume % {"restriction": _choose_map["-"]}
     else:
         error("Unsupported domain_type: %s" % str(domain_type))
     return code
 
-def _generate_circumradius(geometric_dimension, domain_type):
+def _generate_circumradius(tdim, gdim, domain_type):
     "Generate code for computing a cell's circumradius."
 
     # Choose snippets
-    radius = circumradius[geometric_dimension]
+    radius = circumradius[tdim][gdim]
 
     # Choose restrictions
-    if domain_type in ("cell", "exterior_facet"):
+    if domain_type in ("cell", "exterior_facet", "point"):
         code = radius % {"restriction": ""}
     elif domain_type == "interior_facet":
-        code = radius % {"restriction": choose_map["+"]}
-        code += radius % {"restriction": choose_map["-"]}
+        code = radius % {"restriction": _choose_map["+"]}
+        code += radius % {"restriction": _choose_map["-"]}
     else:
         error("Unsupported domain_type: %s" % str(domain_type))
     return code
 
-# Functions.
+def _flatten(i, j, m, n):
+    return i*n + j
+
+# Other functions
+
 def indent(block, num_spaces):
     "Indent each row of the given string block with n spaces."
     indentation = " " * num_spaces
