@@ -16,7 +16,7 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Garth N. Wells, 2009.
-# Modified by Marie Rognes, 2009-2010.
+# Modified by Marie Rognes, 2009-2013.
 # Modified by Martin Alnaes, 2013
 #
 # First added:  2009-03-06
@@ -30,8 +30,10 @@ import ufl
 import FIAT
 
 # FFC modules
-from ffc.log import debug, error
+from ffc.log import debug, error, ffc_assert
 from ffc.quadratureelement import QuadratureElement as FFCQuadratureElement
+from ffc.timeelements import LobattoElement as FFCLobattoElement
+from ffc.timeelements import RadauElement as FFCRadauElement
 
 from ffc.mixedelement import MixedElement
 from ffc.restrictedelement import RestrictedElement
@@ -52,6 +54,21 @@ cellname_to_num_entities = {
     "hexahedron": (8, 12, 6, 1),
     }
 
+# Element families supported by FFC
+supported_families = ("Brezzi-Douglas-Marini",
+                      "Brezzi-Douglas-Fortin-Marini",
+                      "Crouzeix-Raviart",
+                      "Discontinuous Lagrange",
+                      "Lagrange",
+                      "Lobatto",
+                      "Nedelec 1st kind H(curl)",
+                      "Nedelec 2nd kind H(curl)",
+                      "Radau",
+                      "Raviart-Thomas",
+                      "Real",
+                      "Bubble",
+                      "Quadrature")
+
 # Mapping from dimension to number of mesh sub-entities. (In principle,
 # cellname_to_num_entities contains the same information, but with string keys.)
 # DISABLED ON PURPOSE: It's better to use cell name instead of dimension
@@ -116,36 +133,55 @@ def _create_fiat_element(ufl_element):
     cell = ufl_element.cell()
     degree = ufl_element.degree()
 
+    # Check that FFC supports this element
+    ffc_assert(family in supported_families,
+               "This element family (%s) is not supported by FFC." % family)
+
     # Handle the space of the constant
     if family == "Real":
         dg0_element = ufl.FiniteElement("DG", cell, 0)
         constant = _create_fiat_element(dg0_element)
-        return SpaceOfReals(constant)
+        element = SpaceOfReals(constant)
+
+    # Handle the specialized time elements
+    elif family == "Lobatto" :
+        element = FFCLobattoElement(ufl_element.degree())
+
+    elif family == "Radau" :
+        element = FFCRadauElement(ufl_element.degree())
 
     # FIXME: AL: Should this really be here?
     # Handle QuadratureElement
-    if family == "Quadrature":
-        return FFCQuadratureElement(ufl_element)
+    elif family == "Quadrature":
+        element = FFCQuadratureElement(ufl_element)
 
-    # Create FIAT cell
-    fiat_cell = reference_cell(cell.cellname())
-
-    # Handle Bubble element as RestrictedElement of P_{k} to interior
-    if family == "Bubble":
-        V = FIAT.supported_elements["Lagrange"](fiat_cell, degree)
-        dim = cell.geometric_dimension()
-        return RestrictedElement(V, _indices(V, "interior", dim), None)
-
-    # Check if finite element family is supported by FIAT
-    if not family in FIAT.supported_elements:
-        error("Sorry, finite element of type \"%s\" are not supported by FIAT.", family)
-
-    # Create FIAT finite element
-    ElementClass = FIAT.supported_elements[family]
-    if degree is None:
-        element = ElementClass(fiat_cell)
     else:
-        element = ElementClass(fiat_cell, degree)
+        # Create FIAT cell
+        fiat_cell = reference_cell(cell.cellname())
+
+        # Handle Bubble element as RestrictedElement of P_{k} to interior
+        if family == "Bubble":
+            V = FIAT.supported_elements["Lagrange"](fiat_cell, degree)
+            dim = cell.topological_dimension()
+            return RestrictedElement(V, _indices(V, "interior", dim), None)
+
+        # Check if finite element family is supported by FIAT
+        if not family in FIAT.supported_elements:
+            error("Sorry, finite element of type \"%s\" are not supported by FIAT.", family)
+
+        # Create FIAT finite element
+        ElementClass = FIAT.supported_elements[family]
+        if degree is None:
+            element = ElementClass(fiat_cell)
+        else:
+            element = ElementClass(fiat_cell, degree)
+
+    # Consistency check between UFL and FIAT elements. This will not hold for elements
+    # where the reference value shape is different from the global value shape, i.e.
+    # RT elements on a triangle in 3D.
+    #ffc_assert(element.value_shape() == ufl_element.value_shape(),
+    #           "Something went wrong in the construction of FIAT element from UFL element." + \
+    #           "Shapes are %s and %s." % (element.value_shape(), ufl_element.value_shape()))
 
     return element
 
@@ -287,3 +323,4 @@ def _indices(element, domain, dim=0):
 
     else:
         error("Restriction to domain: %s, is not supported." % repr(domain))
+
