@@ -85,6 +85,9 @@ def format_code(code, wrapper_code, prefix, parameters):
             elif "interior_facet_integral" in code_integral["classname"]:
                 code_h += _format_h("interior_facet_integral", code_integral, parameters)
                 code_c += _format_c("interior_facet_integral", code_integral, parameters)
+            elif "point_integral" in code_integral["classname"]:
+                code_h += _format_h("point_integral", code_integral, parameters)
+                code_c += _format_c("point_integral", code_integral, parameters)
 
     # Generate code for form
     if code_forms:
@@ -156,7 +159,7 @@ def _generate_comment(parameters):
 def _generate_additional_includes(codes):
     s = set()
     for code in codes:
-        if code.has_key("additional_includes_set"):
+        if "additional_includes_set" in code:
             s.update(code["additional_includes_set"])
     if s:
         return "\n".join(list(s)) + "\n"
