@@ -1,7 +1,7 @@
 """This module provides a just-in-time (JIT) form compiler.
 It uses Instant to wrap the generated code into a Python module."""
 
-# Copyright (C) 2007-2009 Anders Logg
+# Copyright (C) 2007-2013 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -22,19 +22,23 @@ It uses Instant to wrap the generated code into a Python module."""
 # Modified by Ilmar Wilbers, 2008
 # Modified by Kristian B. Oelgaard, 2009
 # Modified by Joachim Haga, 2011.
+# Modified by Martin Alnaes, 2013
 #
 # First added:  2007-07-20
-# Last changed: 2011-04-26
+# Last changed: 2013-01-25
 
 # Python modules
 import os, sys
 import instant
 import ufc_utils
+import ufc
 
 # UFL modules
 from ufl.classes import Form, FiniteElementBase, TestFunction
+from ufl.domains import as_domain
 from ufl.objects import dx
-from ufl.algorithms import as_form, preprocess, FormData
+from ufl.algorithms import as_form, extract_common_cell, extract_elements, extract_sub_elements
+from ufl.common import istr, tstr
 
 # FFC modules
 from log import log
@@ -49,13 +53,14 @@ from parameters import default_parameters
 from mixedelement import MixedElement
 from compiler import compile_form
 from jitobject import JITObject
+from ffc.quadratureelement import default_quadrature_degree
 
 # Special Options for JIT-compilation
 FFC_PARAMETERS_JIT = default_parameters()
 FFC_PARAMETERS_JIT["no-evaluate_basis_derivatives"] = True
 
 # Set debug level for Instant
-instant.set_logging_level("warning")
+instant.set_log_level("warning")
 
 def jit(ufl_object, parameters=None, common_cell=None):
     """Just-in-time compile the given form or element
@@ -72,6 +77,76 @@ def jit(ufl_object, parameters=None, common_cell=None):
     else:
         return jit_form(ufl_object, parameters, common_cell)
 
+def _auto_select_degree(elements):
+    """
+    Automatically select degree for all elements of the form in cases
+    where this has not been specified by the user. This feature is
+    used by DOLFIN to allow the specification of Expressions with
+    undefined degrees.
+    """
+
+    # Extract common degree
+    common_degree = max([e.degree() for e in elements] or [None])
+    if common_degree is None:
+        common_degree = default_quadrature_degree
+
+    # Degree must be at least 1 (to work with Lagrange elements)
+    common_degree = max(1, common_degree)
+
+    return common_degree
+
+def _compute_element_mapping(form, common_cell):
+    "Compute element mapping for element replacement"
+
+    # Extract all elements
+    elements = extract_elements(form)
+    elements = extract_sub_elements(elements)
+
+    # Get cell and degree
+    # FIXME: implement extract_common_top_domain(s) instead of this
+    common_cell = extract_common_cell(form, common_cell)
+    common_domain = as_domain(common_cell) # FIXME:
+    common_degree = _auto_select_degree(elements)
+
+    # Compute element map
+    element_mapping = {}
+    for element in elements:
+
+        # Flag for whether element needs to be reconstructed
+        reconstruct = False
+
+        # Set cell
+        domain = element.domain()
+        if domain is None:
+            info("Adjusting missing element domain to %s." % \
+                     (common_domain,))
+            domain = common_domain
+            reconstruct = True
+
+        # Set degree
+        degree = element.degree()
+        if degree is None:
+            info("Adjusting element degree from %s to %d" % \
+                     (istr(degree), common_degree))
+            degree = common_degree
+            reconstruct = True
+
+        # Reconstruct element and add to map
+        if reconstruct:
+            element_mapping[element] = element.reconstruct(domain=domain,
+                                                           degree=degree)
+
+    return element_mapping
+
+def check_swig_version(compiled_module):
+    
+    # Check swig version of compiled module
+    if compiled_module and compiled_module.swigversion != ufc.__swigversion__:
+        error("Incompatible swig versions detected. UFC swig "\
+              "version is not the same as extension module swig "\
+              "version: '%s' != '%s' " % \
+              (ufc.__swigversion__, compiled_module.swigversion))
+
 def jit_form(form, parameters=None, common_cell=None):
     "Just-in-time compile the given form."
 
@@ -86,59 +161,60 @@ def jit_form(form, parameters=None, common_cell=None):
     set_level(parameters["log_level"])
     set_prefix(parameters["log_prefix"])
 
+    # Compute element mapping for element replacement
+    element_mapping = _compute_element_mapping(form, common_cell)
+
     # Compute form metadata and extract preprocessed form
-    form_data = form.compute_form_data(common_cell=common_cell)
-    preprocessed_form = form_data.preprocessed_form
+    form_data = form.compute_form_data(common_cell=common_cell,
+                                       element_mapping=element_mapping)
 
     # Wrap input
-    jit_object = JITObject(form, preprocessed_form, parameters, common_cell)
+    jit_object = JITObject(form, parameters)
 
     # Set prefix for generated code
-    prefix = "ffc_form_" + jit_object.signature()
+    module_name = "ffc_form_" + jit_object.signature()
 
     # Use Instant cache if possible
-    cache_dir = parameters["cache_dir"]
-    if cache_dir == "": cache_dir = None
-    module = instant.import_module(jit_object, cache_dir=cache_dir)
+    cache_dir = parameters["cache_dir"] or None
+    module = instant.import_module(module_name, cache_dir=cache_dir)
     if module:
-
+        check_swig_version(module)
         debug("Reusing form from cache.")
-        compiled_form = _extract_form(module, prefix)
-        return (compiled_form, module, form_data, prefix)
+        compiled_form = _extract_form(module, module_name)
+        return (compiled_form, module, form_data, module_name)
 
-    try:
-
-        # Take lock to serialise code generation and compilation.
-        lock = instant.locking.get_lock(instant.get_default_cache_dir(),
-                                        'ffc_' + jit_object.signature())
+    # Take lock to serialise file removal.
+    # Need to add "_0" to lock as instant.import_module acquire
+    # lock with name: module_name
+    with instant.file_lock(instant.get_default_cache_dir(), \
+                           module_name + "_0") as lock:
 
         # Retry Instant cache. The module may have been created while we waited
         # for the lock, even if it didn't exist before.
-        module = instant.import_module(jit_object, cache_dir=cache_dir)
+        module = instant.import_module(module_name, cache_dir=cache_dir)
         if module:
-            compiled_form = _extract_form(module, prefix)
-            return (compiled_form, module, form_data, prefix)
+            check_swig_version(module)
+            debug("Reusing form from cache.")
+            compiled_form = _extract_form(module, module_name)
+            return (compiled_form, module, form_data, module_name)
 
         # Write a message
         log(INFO + 5,
             "Calling FFC just-in-time (JIT) compiler, this may take some time.")
 
         # Generate code
-        compile_form(preprocessed_form,
-                     prefix=prefix,
-                     parameters=parameters,
-                     common_cell=common_cell)
+        compile_form(form,
+                     prefix=module_name,
+                     parameters=parameters)
 
         # Build module using Instant (through UFC)
         debug("Compiling and linking Python extension module, this may take some time.")
-        hfile = prefix + ".h"
-        cppfile = prefix + ".cpp"
+        hfile   = module_name + ".h"
+        cppfile = module_name + ".cpp"
         module = ufc_utils.build_ufc_module(
             hfile,
-            swig_binary=parameters["swig_binary"],
-            swig_path=parameters["swig_path"],
             source_directory = os.curdir,
-            signature = jit_object.signature(),
+            signature = module_name,
             sources = [cppfile] if parameters["split"] else [],
             cppargs = parameters["cpp_optimize_flags"].split() \
                       if parameters["cpp_optimize"] else ["-O0"],
@@ -151,13 +227,12 @@ def jit_form(form, parameters=None, common_cell=None):
             if os.path.isfile(cppfile):
                 os.unlink(cppfile)
 
+        check_swig_version(module)
+
         # Extract compiled form
-        compiled_form = _extract_form(module, prefix)
+        compiled_form = _extract_form(module, module_name)
 
-        return compiled_form, module, form_data, prefix
-
-    finally:
-        instant.locking.release_lock(lock)
+        return compiled_form, module, form_data, module_name
 
 def jit_element(element, parameters=None):
     "Just-in-time compile the given element"
@@ -170,9 +245,8 @@ def jit_element(element, parameters=None):
 
     # Create simplest possible dummy form
     v = TestFunction(element)
-    for i in range(len(element.value_shape())):
-        v = v[i]
-    form = v*dx
+    ii = (0,)*v.rank()
+    form = v[ii]*dx
 
     # Compile form
     compiled_form, module, form_data, prefix = jit_form(form, parameters)
