@@ -9,11 +9,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Garth N. Wells, 2006-2009
 # Modified by Marie E. Rognes, 2007-2010
@@ -71,8 +71,12 @@ class MixedElement:
         """
 
         # Special case: only one element
-        if len(self._elements) == 1:
-            return self._elements[0].tabulate(order, points)
+        # NOTE: KBO: Why do we need this special case? (FFC Bug #798578)
+        #       When calling tabulate() on a MixedElement one should be able to
+        #       rely on getting data back which is ordered like a mixed element
+        #       irrespective of the number of elements?
+#        if len(self._elements) == 1:
+#            return self._elements[0].tabulate(order, points)
 
         # Zeros for insertion into mixed table
         table_shape = (self.space_dimension(), self.num_components(), len(points))
@@ -116,6 +120,10 @@ def _combine_entity_dofs(elements):
     entity_dof containing the information for all the elements.
     """
 
+    # Return {} if no elements
+    if not elements:
+        return {}
+
     # Initialize entity_dofs dictionary
     entity_dofs = dict((key, {}) for key in elements[0].entity_dofs())
     for dim in elements[0].entity_dofs():
