@@ -33,7 +33,7 @@ from ufl.classes import FixedIndex
 from ufl.classes import IntValue
 from ufl.classes import FloatValue
 from ufl.classes import Coefficient
-from ufl.expr import Operator
+from ufl.classes import Operator
 
 # UFL Algorithms.
 from ufl.algorithms.printing import tree_format
@@ -173,17 +173,13 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
             return {(): create_product([val]*expo.value())}
         elif isinstance(expo, FloatValue):
             exp = format["floating point"](expo.value())
-#            sym = create_symbol(format["std power"](str(val), exp), val.t)
-#            sym.base_expr = val
-#            sym.base_op = 1 # Add one operation for the pow() function.
-            sym = create_symbol(format["std power"], val.t, val, 1, exp)
+            sym = create_symbol(format["std power"](str(val), exp), val.t, val, 1)
             return {(): sym}
         elif isinstance(expo, (Coefficient, Operator)):
             exp = self.visit(expo)[()]
-#            sym = create_symbol(format["std power"](str(val), exp[()]), val.t)
-#            sym.base_expr = val
-#            sym.base_op = 1 # Add one operation for the pow() function.
-            sym = create_symbol(format["std power"], val.t, val, 1, exp)
+#            print "pow exp: ", exp
+#            print "pow val: ", val
+            sym = create_symbol(format["std power"](str(val), exp), val.t, val, 1)
             return {(): sym}
         else:
             error("power does not support this exponent: " + repr(expo))
@@ -197,10 +193,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         # Take absolute value of operand.
         val = operands[0][()]
-#        new_val = create_symbol(format["absolute value"](str(val)), val.t)
-#        new_val.base_expr = val
-#        new_val.base_op = 1 # Add one operation for taking the absolute value.
-        new_val = create_symbol(format["absolute value"], val.t, val, 1)
+        new_val = create_symbol(format["absolute value"](str(val)), val.t, val, 1)
         return {():new_val}
 
     # -------------------------------------------------------------------------
@@ -213,7 +206,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         c, = operands
         ffc_assert(len(c) == 1 and c.keys()[0] == (),\
             "Condition for NotCondition should only be one function: " + repr(c))
-        sym = create_symbol("", c[()].t, cond=(c[()], format["not"]))
+        sym = create_symbol(format["not"](str(c[()])), c[()].t, base_op=c[()].ops()+1)
         return {(): sym}
 
     def binary_condition(self, o, *operands):
@@ -234,7 +227,9 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         # Get the minimum type
         t = min(lhs[()].t, rhs[()].t)
-        sym = create_symbol("", t, cond=(lhs[()], format[name_map[o._name]], rhs[()]))
+        ops = lhs[()].ops() + rhs[()].ops() + 1
+        cond = str(lhs[()])+format[name_map[o._name]]+str(rhs[()])
+        sym = create_symbol(format["grouping"](cond), t, base_op=ops)
         return {(): sym}
 
     def conditional(self, o, *operands):
@@ -252,8 +247,11 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         t_val = true[()]
         f_val = false[()]
 
-        # Get the minimum type and number of operations.
-        t = min([cond[()].t, t_val.t, f_val.t])
+        # Get the minimum type and number of operations
+        # TODO: conditionals are currently always located inside the ip loop,
+        # therefore the type has to be at least IP (fix bug #1082048). This can
+        # be optimised.
+        t = min([cond[()].t, t_val.t, f_val.t, IP])
         ops = sum([cond[()].ops(), t_val.ops(), f_val.ops()])
 
         # Create expression for conditional
@@ -280,28 +278,22 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         # Safety check.
         ffc_assert(not operands, "Didn't expect any operands for FacetNormal: " + repr(operands))
+        ffc_assert(len(components) == 1,
+                   "FacetNormal expects 1 component index: " + repr(components))
 
         # Handle 1D as a special case.
         # FIXME: KBO: This has to change for mD elements in R^n : m < n
-        if self.geo_dim == 1:
-            # Safety check.
-            ffc_assert(len(components) == 0, "FacetNormal in 1D does not expect a component index: " + repr(components))
+        if self.gdim == 1: # FIXME: MSA UFL uses shape (1,) now, can we remove the special case here then?
             normal_component = format["normal component"](self.restriction, "")
-            self.trans_set.add(normal_component)
         else:
-
-            # Safety check.
-            ffc_assert(len(components) == 1, "FacetNormal expects 1 component index: " + repr(components))
-
-            # We get one component.
             normal_component = format["normal component"](self.restriction, components[0])
-            self.trans_set.add(normal_component)
+        self.trans_set.add(normal_component)
 
         return {(): create_symbol(normal_component, GEO)}
 
     def cell_volume(self, o,  *operands):
         # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for FacetNormal: " + repr(operands))
+        ffc_assert(not operands, "Didn't expect any operands for CellVolume: " + repr(operands))
 
         # FIXME: KBO: This has to change for higher order elements
 #        detJ = format["det(J)"](self.restriction)
@@ -323,9 +315,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         return {():create_symbol(circumradius, GEO)}
 
-    def facet_area(self, o,  *operands):
-        # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for FacetArea: " + repr(operands))
+    def facet_area(self, o):
 
         # FIXME: KBO: This has to change for higher order elements
         # NOTE: Omitting restriction because the area of a facet is the same
@@ -337,62 +327,90 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         return {():create_symbol(area, GEO)}
 
+    def min_facet_edge_length(self, o):
+        # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
+
+        if self.tdim < 3:
+            return self.facet_area(o)
+
+        edgelen = format["min facet edge length"](self.restriction)
+        self.trans_set.add(edgelen)
+
+        return {():create_symbol(edgelen, GEO)}
+
+    def max_facet_edge_length(self, o):
+        # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
+
+        if self.tdim < 3:
+            return self.facet_area(o)
+
+        edgelen = format["max facet edge length"](self.restriction)
+        self.trans_set.add(edgelen)
+
+        return {():create_symbol(edgelen, GEO)}
+
     # -------------------------------------------------------------------------
 
     def create_argument(self, ufl_argument, derivatives, component, local_comp,
-                  local_offset, ffc_element, transformation, multiindices):
+                        local_offset, ffc_element, transformation, multiindices,
+                        tdim, gdim, avg):
         "Create code for basis functions, and update relevant tables of used basis."
+        ffc_assert(ufl_argument in self._function_replace_values, "Expecting ufl_argument to have been mapped prior to this call.")
 
         # Prefetch formats to speed up code generation.
         f_transform     = format["transform"]
         f_detJ          = format["det(J)"]
 
+        # Reset code
         code = {}
 
-        # Affince mapping
+        # Affine mapping
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
 
-                # Call function to create mapping and basis name.
-                mapping, basis = self._create_mapping_basis(component, deriv, ufl_argument, ffc_element)
+                # Create mapping and basis name.
+                mapping, basis = self._create_mapping_basis(component, deriv, avg, ufl_argument, ffc_element)
+                if not mapping in code:
+                    code[mapping] = []
 
-                # Add transformation if needed.
-                if mapping in code:
-                    code[mapping].append(self.__apply_transform(basis, derivatives, multi))
-                else:
-                    code[mapping] = [self.__apply_transform(basis, derivatives, multi)]
+                if basis is not None:
+                    # Add transformation if needed.
+                    code[mapping].append(self.__apply_transform(basis, derivatives, multi, tdim, gdim))
 
         # Handle non-affine mappings.
         else:
+            ffc_assert(avg is None, "Taking average is not supported for non-affine mappings.")
+
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                for c in range(self.geo_dim):
-                    # Call function to create mapping and basis name.
-                    mapping, basis = self._create_mapping_basis(c + local_offset, deriv, ufl_argument, ffc_element)
 
-                    # Multiply basis by appropriate transform.
-                    if transformation == "covariant piola":
-                        dxdX = create_symbol(f_transform("JINV", c, local_comp, self.restriction), GEO)
-                        basis = create_product([dxdX, basis])
-                    elif transformation == "contravariant piola":
-                        detJ = create_fraction(create_float(1), create_symbol(f_detJ(self.restriction), GEO))
-                        dXdx = create_symbol(f_transform("J", local_comp, c, self.restriction), GEO)
-                        basis = create_product([detJ, dXdx, basis])
-                    else:
-                        error("Transformation is not supported: " + repr(transformation))
+                for c in range(self.tdim):
+                    # Create mapping and basis name.
+                    mapping, basis = self._create_mapping_basis(c + local_offset, deriv, avg, ufl_argument, ffc_element)
+                    if not mapping in code:
+                        code[mapping] = []
 
-                    # Add transformation if needed.
-                    if mapping in code:
-                        code[mapping].append(self.__apply_transform(basis, derivatives, multi))
-                    else:
-                        code[mapping] = [self.__apply_transform(basis, derivatives, multi)]
+                    if basis is not None:
+                        # Multiply basis by appropriate transform.
+                        if transformation == "covariant piola":
+                            dxdX = create_symbol(f_transform("JINV", c, local_comp, tdim, gdim, self.restriction), GEO)
+                            basis = create_product([dxdX, basis])
+                        elif transformation == "contravariant piola":
+                            detJ = create_fraction(create_float(1), create_symbol(f_detJ(self.restriction), GEO))
+                            dXdx = create_symbol(f_transform("J", local_comp, c, gdim, tdim, self.restriction), GEO)
+                            basis = create_product([detJ, dXdx, basis])
+                        else:
+                            error("Transformation is not supported: " + repr(transformation))
+
+                        # Add transformation if needed.
+                        code[mapping].append(self.__apply_transform(basis, derivatives, multi, tdim, gdim))
 
         # Add sums and group if necessary.
         for key, val in code.items():
@@ -404,53 +422,59 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         return code
 
     def create_function(self, ufl_function, derivatives, component, local_comp,
-                  local_offset, ffc_element, quad_element, transformation, multiindices):
+                       local_offset, ffc_element, is_quad_element, transformation, multiindices,
+                       tdim, gdim, avg):
         "Create code for basis functions, and update relevant tables of used basis."
+        ffc_assert(ufl_function in self._function_replace_values, "Expecting ufl_function to have been mapped prior to this call.")
 
         # Prefetch formats to speed up code generation.
         f_transform     = format["transform"]
         f_detJ          = format["det(J)"]
 
+        # Reset code
         code = []
 
         # Handle affine mappings.
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                # Call other function to create function name.
-                function_name = self._create_function_name(component, deriv, quad_element, ufl_function, ffc_element)
-                if not function_name:
-                    continue
 
-                # Add transformation if needed.
-                code.append(self.__apply_transform(function_name, derivatives, multi))
+                # Create function name.
+                function_name = self._create_function_name(component, deriv, avg, is_quad_element, ufl_function, ffc_element)
+                if function_name:
+                    # Add transformation if needed.
+                    code.append(self.__apply_transform(function_name, derivatives, multi, tdim, gdim))
 
         # Handle non-affine mappings.
         else:
+            ffc_assert(avg is None, "Taking average is not supported for non-affine mappings.")
+
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                for c in range(self.geo_dim):
-                    function_name = self._create_function_name(c + local_offset, deriv, quad_element, ufl_function, ffc_element)
 
-                    # Multiply basis by appropriate transform.
-                    if transformation == "covariant piola":
-                        dxdX = create_symbol(f_transform("JINV", c, local_comp, self.restriction), GEO)
-                        function_name = create_product([dxdX, function_name])
-                    elif transformation == "contravariant piola":
-                        detJ = create_fraction(create_float(1), create_symbol(f_detJ(self.restriction), GEO))
-                        dXdx = create_symbol(f_transform("J", local_comp, c, self.restriction), GEO)
-                        function_name = create_product([detJ, dXdx, function_name])
-                    else:
-                        error("Transformation is not supported: ", repr(transformation))
+                for c in range(self.tdim):
+                    function_name = self._create_function_name(c + local_offset, deriv, avg, is_quad_element, ufl_function, ffc_element)
+                    if function_name:
+                        # Multiply basis by appropriate transform.
+                        if transformation == "covariant piola":
+                            dxdX = create_symbol(f_transform("JINV", c, local_comp, tdim, gdim, self.restriction), GEO)
+                            function_name = create_product([dxdX, function_name])
+                        elif transformation == "contravariant piola":
+                            detJ = create_fraction(create_float(1), create_symbol(f_detJ(self.restriction), GEO))
+                            dXdx = create_symbol(f_transform("J", local_comp, c, gdim, tdim, self.restriction), GEO)
+                            function_name = create_product([detJ, dXdx, function_name])
+                        else:
+                            error("Transformation is not supported: ", repr(transformation))
+
+                        # Add transformation if needed.
+                        code.append(self.__apply_transform(function_name, derivatives, multi, tdim, gdim))
 
-                    # Add transformation if needed.
-                    code.append(self.__apply_transform(function_name, derivatives, multi))
         if not code:
             return create_float(0.0)
         elif len(code) > 1:
@@ -463,7 +487,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # Helper functions for Argument and Coefficient
     # -------------------------------------------------------------------------
-    def __apply_transform(self, function, derivatives, multi):
+    def __apply_transform(self, function, derivatives, multi, tdim, gdim):
         "Apply transformation (from derivatives) to basis or function."
         f_transform     = format["transform"]
 
@@ -471,7 +495,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         transforms = []
         for i, direction in enumerate(derivatives):
             ref = multi[i]
-            t = f_transform("JINV", ref, direction, self.restriction)
+            t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
             transforms.append(create_symbol(t, GEO))
         transforms.append(function)
         return create_product(transforms)
@@ -499,11 +523,9 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         # Use format function on value of operand.
         operand = operands[0]
         for key, val in operand.items():
-#            new_val = create_symbol(format_function(str(val)), val.t)
-#            new_val.base_expr = val
-#            new_val.base_op = 1 # Add one operation for the math function.
-            new_val = create_symbol(format_function, val.t, val, 1)
+            new_val = create_symbol(format_function(str(val)), val.t, val, 1)
             operand[key] = new_val
+        #raise Exception("pause")
         return operand
 
     def _bessel_function(self, operands, format_function):
@@ -523,7 +545,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         if x is None:
             x = format["floating point"](0.0)
 
-        sym = create_symbol(format_function, x.t, x, 1, nu)
+        sym = create_symbol(format_function(x,nu), x.t, x, 1)
         return {():sym}
 
     # -------------------------------------------------------------------------
@@ -532,7 +554,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     def _count_operations(self, expression):
         return expression.ops()
 
-    def _create_entry_data(self, val):
+    def _create_entry_data(self, val, domain_type):
 #        zero = False
         # Multiply value by weight and determinant
         ACCESS = GEO
@@ -541,18 +563,23 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
             weight += format["component"]("", format["integration points"])
             ACCESS = IP
         weight = self._create_symbol(weight, ACCESS)[()]
-        f_scale_factor = format["scale factor"]
 
         # Create value.
-        value = create_product([val, weight, create_symbol(f_scale_factor, GEO)])
+        if domain_type == "point":
+            trans_set = set()
+            value = create_product([val, weight])
+        else:
+            f_scale_factor = format["scale factor"]
+            trans_set = set([f_scale_factor])
+            value = create_product([val, weight,
+                                    create_symbol(f_scale_factor, GEO)])
 
         # Update sets of used variables (if they will not be used because of
         # optimisations later, they will be reset).
-        trans_set = set([f_scale_factor])
         trans_set.update(map(lambda x: str(x), value.get_unique_vars(GEO)))
         used_points = set([self.points])
         ops = self._count_operations(value)
-        used_psi_tables = set([self.psi_tables_map[b] for b in value.get_unique_vars(BASIS)])
+        used_psi_tables = set([self.psi_tables_map[b]
+                               for b in value.get_unique_vars(BASIS)])
 
         return (value, ops, [trans_set, used_points, used_psi_tables])
-
