@@ -17,11 +17,11 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Peter Brune, 2009
-# Modified by Anders Logg, 2009
+# Modified by Peter Brune 2009
+# Modified by Anders Logg 2009, 2013
 #
 # First added:  2009-02-09
-# Last changed: 2011-11-22
+# Last changed: 2013-01-10
 
 # Python modules.
 from numpy import shape
@@ -34,7 +34,7 @@ from ufl.classes import FixedIndex
 from ufl.classes import IntValue
 from ufl.classes import FloatValue
 from ufl.classes import Coefficient
-from ufl.expr import Operator
+from ufl.classes import Operator
 
 # UFL Algorithms.
 from ufl.algorithms.printing import tree_format
@@ -316,7 +316,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         if r_val is None:
             r_val = format["float"](0.0)
 
-        return {(): l_val + format[name_map[o._name]] + r_val}
+        return {(): format["grouping"](l_val + format[name_map[o._name]] + r_val)}
 
     def conditional(self, o, *operands):
 
@@ -351,57 +351,40 @@ class QuadratureTransformer(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # FacetNormal, CellVolume, Circumradius, FacetArea (geometry.py).
     # -------------------------------------------------------------------------
-    def facet_normal(self, o,  *operands):
+    def facet_normal(self, o):
         #print("Visiting FacetNormal:")
 
         # Get the component
         components = self.component()
 
         # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for FacetNormal: " + repr(operands))
+        ffc_assert(len(components) == 1, "FacetNormal expects 1 component index: " + repr(components))
 
         # Handle 1D as a special case.
         # FIXME: KBO: This has to change for mD elements in R^n : m < n
-        if self.geo_dim == 1:
-            # Safety check.
-            ffc_assert(len(components) == 0, "FacetNormal in 1D does not expect a component index: " + repr(components))
+        if self.gdim == 1: # FIXME: MSA: UFL uses shape (1,) now, can we remove the special case here then?
             normal_component = format["normal component"](self.restriction, "")
-            self.trans_set.add(normal_component)
         else:
-
-            # Safety check.
-            ffc_assert(len(components) == 1, "FacetNormal expects 1 component index: " + repr(components))
-
-            # We get one component.
             normal_component = format["normal component"](self.restriction, components[0])
-            self.trans_set.add(normal_component)
+        self.trans_set.add(normal_component)
 
         return {():normal_component}
 
-    def cell_volume(self, o,  *operands):
-        # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for CellVolume: " + repr(operands))
-
+    def cell_volume(self, o):
         # FIXME: KBO: This has to change for higher order elements
         volume = format["cell volume"](self.restriction)
         self.trans_set.add(volume)
 
         return {():volume}
 
-    def circumradius(self, o,  *operands):
-        # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for Circumradius: " + repr(operands))
-
+    def circumradius(self, o):
         # FIXME: KBO: This has to change for higher order elements
         circumradius = format["circumradius"](self.restriction)
         self.trans_set.add(circumradius)
 
         return {():circumradius}
 
-    def facet_area(self, o,  *operands):
-        # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for FacetArea: " + repr(operands))
-
+    def facet_area(self, o):
         # FIXME: KBO: This has to change for higher order elements
         # NOTE: Omitting restriction because the area of a facet is the same
         # on both sides.
@@ -412,11 +395,35 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         return {():area}
 
+    def min_facet_edge_length(self, o):
+        # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
+
+        if self.tdim < 3:
+            return self.facet_area(o)
+
+        edgelen = format["min facet edge length"](self.restriction)
+        self.trans_set.add(edgelen)
+
+        return {():edgelen}
+
+    def max_facet_edge_length(self, o):
+        # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
+
+        if self.tdim < 3:
+            return self.facet_area(o)
+
+        edgelen = format["max facet edge length"](self.restriction)
+        self.trans_set.add(edgelen)
+
+        return {():edgelen}
+
     # -------------------------------------------------------------------------
 
     def create_argument(self, ufl_argument, derivatives, component, local_comp,
-                  local_offset, ffc_element, transformation, multiindices):
+                        local_offset, ffc_element, transformation, multiindices,
+                        tdim, gdim, avg):
         "Create code for basis functions, and update relevant tables of used basis."
+        ffc_assert(ufl_argument in self._function_replace_values, "Expecting ufl_argument to have been mapped prior to this call.")
 
         # Prefetch formats to speed up code generation.
         f_group         = format["grouping"]
@@ -426,61 +433,60 @@ class QuadratureTransformer(QuadratureTransformerBase):
         f_detJ          = format["det(J)"]
         f_inv           = format["inverse"]
 
+        # Reset code
         code = {}
+
         # Handle affine mappings.
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                # Call function to create mapping and basis name.
-                mapping, basis = self._create_mapping_basis(component, deriv, ufl_argument, ffc_element)
-                if basis is None:
-                    if not mapping in code:
-                        code[mapping] = []
-                    continue
 
-                # Add transformation if needed.
-                if mapping in code:
-                    code[mapping].append(self.__apply_transform(basis, derivatives, multi))
-                else:
-                    code[mapping] = [self.__apply_transform(basis, derivatives, multi)]
+                # Create mapping and basis name.
+                #print "component = ", component
+                mapping, basis = self._create_mapping_basis(component, deriv, avg, ufl_argument, ffc_element)
+                if not mapping in code:
+                    code[mapping] = []
+
+                if basis is not None:
+                    # Add transformation
+                    code[mapping].append(self.__apply_transform(basis, derivatives, multi, tdim, gdim))
 
         # Handle non-affine mappings.
         else:
+            ffc_assert(avg is None, "Taking average is not supported for non-affine mappings.")
+
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                for c in range(self.geo_dim):
-                    # Call function to create mapping and basis name.
-                    mapping, basis = self._create_mapping_basis(c + local_offset, deriv, ufl_argument, ffc_element)
-                    if basis is None:
-                        if not mapping in code:
-                            code[mapping] = []
-                        continue
 
-                    # Multiply basis by appropriate transform.
-                    if transformation == "covariant piola":
-                        dxdX = f_transform("JINV", c, local_comp, self.restriction)
-                        self.trans_set.add(dxdX)
-                        basis = f_mult([dxdX, basis])
-                    elif transformation == "contravariant piola":
-                        self.trans_set.add(f_detJ(self.restriction))
-                        detJ = f_inv(f_detJ(self.restriction))
-                        dXdx = f_transform("J", local_comp, c, self.restriction)
-                        self.trans_set.add(dXdx)
-                        basis = f_mult([detJ, dXdx, basis])
-                    else:
-                        error("Transformation is not supported: " + repr(transformation))
+                for c in range(self.tdim):
+                    # Create mapping and basis name.
+                    mapping, basis = self._create_mapping_basis(c + local_offset, deriv, avg, ufl_argument, ffc_element)
+                    if not mapping in code:
+                        code[mapping] = []
 
-                    # Add transformation if needed.
-                    if mapping in code:
-                        code[mapping].append(self.__apply_transform(basis, derivatives, multi))
-                    else:
-                        code[mapping] = [self.__apply_transform(basis, derivatives, multi)]
+                    if basis is not None:
+                        # Multiply basis by appropriate transform.
+                        if transformation == "covariant piola":
+                            dxdX = f_transform("JINV", c, local_comp, tdim, gdim, self.restriction)
+                            self.trans_set.add(dxdX)
+                            basis = f_mult([dxdX, basis])
+                        elif transformation == "contravariant piola":
+                            self.trans_set.add(f_detJ(self.restriction))
+                            detJ = f_inv(f_detJ(self.restriction))
+                            dXdx = f_transform("J", local_comp, c, gdim, tdim, self.restriction)
+                            self.trans_set.add(dXdx)
+                            basis = f_mult([detJ, dXdx, basis])
+                        else:
+                            error("Transformation is not supported: " + repr(transformation))
+
+                        # Add transformation if needed.
+                        code[mapping].append(self.__apply_transform(basis, derivatives, multi, tdim, gdim))
 
         # Add sums and group if necessary.
         for key, val in code.items():
@@ -495,8 +501,10 @@ class QuadratureTransformer(QuadratureTransformerBase):
         return code
 
     def create_function(self, ufl_function, derivatives, component, local_comp,
-                  local_offset, ffc_element, quad_element, transformation, multiindices):
+                        local_offset, ffc_element, is_quad_element,
+                        transformation, multiindices, tdim, gdim, avg):
         "Create code for basis functions, and update relevant tables of used basis."
+        ffc_assert(ufl_function in self._function_replace_values, "Expecting ufl_function to have been mapped prior to this call.")
 
         # Prefetch formats to speed up code generation.
         f_mult          = format["multiply"]
@@ -504,50 +512,52 @@ class QuadratureTransformer(QuadratureTransformerBase):
         f_detJ          = format["det(J)"]
         f_inv           = format["inverse"]
 
+        # Reset code
         code = []
+
         # Handle affine mappings.
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                # Call other function to create function name.
-                function_name = self._create_function_name(component, deriv, quad_element, ufl_function, ffc_element)
-                if function_name is None:
-                    continue
 
-                # Add transformation if needed.
-                code.append(self.__apply_transform(function_name, derivatives, multi))
+                # Create function name.
+                function_name = self._create_function_name(component, deriv, avg, is_quad_element, ufl_function, ffc_element)
+                if function_name:
+                    # Add transformation if needed.
+                    code.append(self.__apply_transform(function_name, derivatives, multi, tdim, gdim))
 
         # Handle non-affine mappings.
         else:
+            ffc_assert(avg is None, "Taking average is not supported for non-affine mappings.")
+
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.geo_dim)]
+                deriv = [multi.count(i) for i in range(self.tdim)]
                 if not any(deriv):
                     deriv = []
-                for c in range(self.geo_dim):
-                    function_name = self._create_function_name(c + local_offset, deriv, quad_element, ufl_function, ffc_element)
-                    if function_name is None:
-                        continue
 
-                    # Multiply basis by appropriate transform.
-                    if transformation == "covariant piola":
-                        dxdX = f_transform("JINV", c, local_comp, self.restriction)
-                        self.trans_set.add(dxdX)
-                        function_name = f_mult([dxdX, function_name])
-                    elif transformation == "contravariant piola":
-                        self.trans_set.add(f_detJ(self.restriction))
-                        detJ = f_inv(f_detJ(self.restriction))
-                        dXdx = f_transform("J", local_comp, c, self.restriction)
-                        self.trans_set.add(dXdx)
-                        function_name = f_mult([detJ, dXdx, function_name])
-                    else:
-                        error("Transformation is not supported: ", repr(transformation))
+                for c in range(self.tdim):
+                    function_name = self._create_function_name(c + local_offset, deriv, avg, is_quad_element, ufl_function, ffc_element)
+                    if function_name:
+                        # Multiply basis by appropriate transform.
+                        if transformation == "covariant piola":
+                            dxdX = f_transform("JINV", c, local_comp, tdim, gdim, self.restriction)
+                            self.trans_set.add(dxdX)
+                            function_name = f_mult([dxdX, function_name])
+                        elif transformation == "contravariant piola":
+                            self.trans_set.add(f_detJ(self.restriction))
+                            detJ = f_inv(f_detJ(self.restriction))
+                            dXdx = f_transform("J", local_comp, c, gdim, tdim, self.restriction)
+                            self.trans_set.add(dXdx)
+                            function_name = f_mult([detJ, dXdx, function_name])
+                        else:
+                            error("Transformation is not supported: ", repr(transformation))
 
-                    # Add transformation if needed.
-                    code.append(self.__apply_transform(function_name, derivatives, multi))
+                        # Add transformation if needed.
+                        code.append(self.__apply_transform(function_name, derivatives, multi, tdim, gdim))
 
         if not code:
             return None
@@ -561,7 +571,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # Helper functions for Argument and Coefficient
     # -------------------------------------------------------------------------
-    def __apply_transform(self, function, derivatives, multi):
+    def __apply_transform(self, function, derivatives, multi, tdim, gdim): # XXX UFLACS REUSE
         "Apply transformation (from derivatives) to basis or function."
         f_mult          = format["multiply"]
         f_transform     = format["transform"]
@@ -570,7 +580,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         transforms = []
         for i, direction in enumerate(derivatives):
             ref = multi[i]
-            t = f_transform("JINV", ref, direction, self.restriction)
+            t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
             self.trans_set.add(t)
             transforms.append(t)
 
@@ -609,6 +619,11 @@ class QuadratureTransformer(QuadratureTransformerBase):
             new_operand[key] = format_function(val)
         return new_operand
 
+    def _atan_2_function(self, operands, format_function):
+        x1,x2 = operands
+        x1,x2 = x1.values()[0],x2.values()[0]
+        return {():format_function(x1, x2)}
+
     def _bessel_function(self, operands, format_function):
         # TODO: Are these safety checks needed?
         ffc_assert(len(operands) == 2,\
@@ -637,19 +652,24 @@ class QuadratureTransformer(QuadratureTransformerBase):
     def _count_operations(self, expression):
         return operation_count(expression, format)
 
-    def _create_entry_data(self, val):
+    def _create_entry_data(self, val, domain_type):
         # Multiply value by weight and determinant
         # Create weight and scale factor.
         weight = format["weight"](self.points)
         if self.points > 1:
             weight += format["component"]("", format["integration points"])
-        f_scale_factor = format["scale factor"]
 
         # Update sets of used variables.
-        trans_set = set([f_scale_factor])
+        if domain_type == "point":
+            trans_set = set()
+            value = format["mul"]([val, weight])
+        else:
+            f_scale_factor = format["scale factor"]
+            trans_set = set([f_scale_factor])
+            value = format["mul"]([val, weight, f_scale_factor])
+
         trans_set.update(self.trans_set)
         used_points = set([self.points])
-        value = format["mul"]([val, weight, f_scale_factor])
         ops = self._count_operations(value)
         used_psi_tables = set([v for k, v in self.psi_tables_map.items()])
 
