@@ -26,17 +26,15 @@ transformers to translate UFL expressions."""
 
 # Python modules.
 from itertools import izip
-import time
 from numpy import shape, array
 
 # UFL Classes.
-from ufl.classes import MultiIndex, FixedIndex, Index
-from ufl.common import StackDict, Stack
+from ufl.classes import FixedIndex, Index
+from ufl.common import StackDict, Stack, product
 from ufl.permutation import build_component_numbering
 
 # UFL Algorithms.
-from ufl.algorithms import Transformer, tree_format
-from ufl.algorithms import strip_variables
+from ufl.algorithms import Transformer
 
 # FFC modules.
 from ffc.log import ffc_assert, error, info
@@ -50,7 +48,6 @@ from ffc.representationutils import transform_component
 
 # Utility and optimisation functions for quadraturegenerator.
 from quadratureutils import create_psi_tables
-from symbolics import generate_aux_constants
 from symbolics import BASIS, IP, GEO, CONST
 
 class QuadratureTransformerBase(Transformer):
@@ -70,7 +67,10 @@ class QuadratureTransformerBase(Transformer):
         # Save optimise_parameters, weights and fiat_elements_map.
         self.optimise_parameters = optimise_parameters
 
+        # Map from original functions with possibly incomplete elements
+        # to functions with properly completed elements
         self._function_replace_map = function_replace_map
+        self._function_replace_values = set(function_replace_map.values()) # For assertions
 
         # Create containers and variables.
         self.used_psi_tables = set()
@@ -89,10 +89,11 @@ class QuadratureTransformerBase(Transformer):
         self.facet1 = None
         self.vertex = None
         self.restriction = None
+        self.avg = None
         self.coordinate = None
         self.conditionals = {}
         self.additional_includes_set = set()
-        self.__psi_tables = psi_tables
+        self.__psi_tables = psi_tables # TODO: Unused? Remove?
 
         # Stacks.
         self._derivatives = []
@@ -168,12 +169,12 @@ class QuadratureTransformerBase(Transformer):
     # Nothing in expr.py is handled. Can only handle children of these clases.
     def expr(self, o):
         print "\n\nVisiting basic Expr:", repr(o), "with operands:"
-        error("This expression is not handled: ", repr(o))
+        error("This expression is not handled: " + repr(o))
 
     # Nothing in terminal.py is handled. Can only handle children of these clases.
     def terminal(self, o):
         print "\n\nVisiting basic Terminal:", repr(o), "with operands:"
-        error("This terminal is not handled: ", repr(o))
+        error("This terminal is not handled: " + repr(o))
 
     # -------------------------------------------------------------------------
     # Things which should not be here (after expansion etc.) from:
@@ -225,6 +226,10 @@ class QuadratureTransformerBase(Transformer):
         print "\n\nVisiting MathFunction:", repr(o)
         error("This MathFunction is not supported (yet).")
 
+    def atan_2_function(self, o):
+        print "\n\nVisiting Atan2Function:", repr(o)
+        error("Atan2Function is not implemented (yet).")
+
     def bessel_function(self, o):
         print "\n\nVisiting BesselFunction:", repr(o)
         error("BesselFunction is not implemented (yet).")
@@ -291,23 +296,21 @@ class QuadratureTransformerBase(Transformer):
         derivatives = self.derivatives()
 
         # Check if basis is already in cache
-        basis = self.argument_cache.get((o, components, derivatives, self.restriction), None)
+        key = (o, components, derivatives, self.restriction, self.avg)
+        basis = self.argument_cache.get(key, None)
+
         # FIXME: Why does using a code dict from cache make the expression manipulations blow (MemoryError) up later?
-        if basis is not None and not self.optimise_parameters["optimisation"]:
-        #if basis is not None:
-            return basis
+        if basis is None or self.optimise_parameters["optimisation"]:
+            # Get auxiliary variables to generate basis
+            (component, local_elem, local_comp, local_offset,
+             ffc_element, transformation, multiindices) = self._get_auxiliary_variables(o, components, derivatives)
 
-        # Get auxiliary variables to generate basis
-        (component, local_elem, local_comp, local_offset,
-         ffc_element, transformation, multiindices) = self._get_auxiliary_variables(o, components, derivatives)
-
-        # Create mapping and code for basis function and add to dict.
-        basis = self.create_argument(o, derivatives, component, local_comp,
-                                     local_offset, ffc_element,
-                                     transformation, multiindices,
-                                     self.tdim, self.gdim)
-
-        self.argument_cache[(o, components, derivatives, self.restriction)] = basis
+            # Create mapping and code for basis function and add to dict.
+            basis = self.create_argument(o, derivatives, component, local_comp,
+                                         local_offset, ffc_element,
+                                         transformation, multiindices,
+                                         self.tdim, self.gdim, self.avg)
+            self.argument_cache[key] = basis
 
         return basis
 
@@ -359,7 +362,7 @@ class QuadratureTransformerBase(Transformer):
             # This happens in 1D, sligtly messy result of defining grad(f) == f.dx(0)
             der = 0
         else:
-            ffc_error("Unexpected rank %d and component length %d in grad expression." % (en, cn))
+            error("Unexpected rank %d and component length %d in grad expression." % (en, cn))
 
         # Add direction to list of derivatives
         self._derivatives.append(der)
@@ -387,7 +390,7 @@ class QuadratureTransformerBase(Transformer):
         derivatives = self.derivatives()
 
         # Check if function is already in cache
-        key = (o, components, derivatives, self.restriction)
+        key = (o, components, derivatives, self.restriction, self.avg)
         function_code = self.function_cache.get(key)
 
         # FIXME: Why does using a code dict from cache make the expression manipulations blow (MemoryError) up later?
@@ -404,7 +407,7 @@ class QuadratureTransformerBase(Transformer):
             # Create code for function and add empty tuple to cache dict.
             function_code = {(): self.create_function(o, derivatives, component,
                                                       local_comp, local_offset, ffc_element, is_quad_element,
-                                                      transformation, multiindices, self.tdim, self.gdim)}
+                                                      transformation, multiindices, self.tdim, self.gdim, self.avg)}
 
             self.function_cache[key] = function_code
 
@@ -605,6 +608,11 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting Atan: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
         return self._math_function(operands, format["atan"])
 
+    def atan_2(self, o, *operands):
+        #print("\n\nVisiting Atan2: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        self.additional_includes_set.add("#include <cmath>")
+        return self._atan_2_function(operands, format["atan_2"])
+
     def erf(self, o, *operands):
         #print("\n\nVisiting Erf: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
         return self._math_function(operands, format["erf"])
@@ -666,6 +674,33 @@ class QuadratureTransformerBase(Transformer):
 
         return code
 
+    def cell_avg(self, o):
+        ffc_assert(self.avg is None, "Not expecting nested averages.")
+
+        # Just get the first operand, there should only be one.
+        expr, = o.operands()
+
+        # Set average marker, visit operand and reset marker
+        self.avg = "cell"
+        code = self.visit(expr)
+        self.avg = None
+
+        return code
+
+    def facet_avg(self, o):
+        ffc_assert(self.avg is None, "Not expecting nested averages.")
+        ffc_assert(self.entitytype != "cell", "Cannot take facet_avg in a cell integral.")
+
+        # Just get the first operand, there should only be one.
+        expr, = o.operands()
+
+        # Set average marker, visit operand and reset marker
+        self.avg = "facet"
+        code = self.visit(expr)
+        self.avg = None
+
+        return code
+
     # -------------------------------------------------------------------------
     # ComponentTensor (tensors.py).
     # -------------------------------------------------------------------------
@@ -830,6 +865,9 @@ class QuadratureTransformerBase(Transformer):
     def _math_function(self, operands, format_function):
         error("This function should be implemented by the child class.")
 
+    def _atan_2_function(self, operands, format_function):
+        error("This function should be implemented by the child class.")
+
     def _get_auxiliary_variables(self,
                                  ufl_function,
                                  component,
@@ -887,11 +925,12 @@ class QuadratureTransformerBase(Transformer):
         else:
             error("Unknown entity type %s." % self.entitytype)
 
-    def _create_mapping_basis(self, component, deriv, ufl_argument, ffc_element):
+    def _create_mapping_basis(self, component, deriv, avg, ufl_argument, ffc_element):
         "Create basis name and mapping from given basis_info."
+        ffc_assert(ufl_argument in self._function_replace_values, "Expecting ufl_argument to have been mapped prior to this call.")
 
         # Get string for integration points.
-        f_ip = format["integration points"]
+        f_ip = "0" if (avg or self.points == 1) else format["integration points"]
         generate_psi_name = format["psi name"]
 
         # Only support test and trial functions.
@@ -903,13 +942,11 @@ class QuadratureTransformerBase(Transformer):
                    "Currently, Argument index must be either 0 or 1: " + repr(ufl_argument))
 
         # Get element counter and loop index.
-        element_counter = self.element_map[self.points][ufl_argument.element()]
+        element_counter = self.element_map[1 if avg else self.points][ufl_argument.element()]
         loop_index = indices[ufl_argument.count()]
 
         # Create basis access, we never need to map the entry in the basis table
         # since we will either loop the entire space dimension or the non-zeros.
-        if self.points == 1:
-            f_ip = "0"
         basis_access = format["component"]("", [f_ip, loop_index])
 
         # Offset element space dimension in case of negative restriction,
@@ -924,7 +961,7 @@ class QuadratureTransformerBase(Transformer):
         # Get current cell entity, with current restriction considered
         entity = self._get_current_entity()
 
-        name = generate_psi_name(element_counter, self.entitytype, entity, component, deriv)
+        name = generate_psi_name(element_counter, self.entitytype, entity, component, deriv, avg)
         name, non_zeros, zeros, ones = self.name_map[name]
         loop_index_range = shape(self.unique_tables[name])[1]
 
@@ -965,13 +1002,14 @@ class QuadratureTransformerBase(Transformer):
 
         return (mapping, basis)
 
-    def _create_function_name(self, component, deriv, is_quad_element, ufl_function, ffc_element):
+    def _create_function_name(self, component, deriv, avg, is_quad_element, ufl_function, ffc_element):
+        ffc_assert(ufl_function in self._function_replace_values, "Expecting ufl_function to have been mapped prior to this call.")
 
         # Get string for integration points.
-        f_ip = "0" if self.points == 1 else format["integration points"]
+        f_ip = "0" if (avg or self.points == 1) else format["integration points"]
 
         # Get the element counter.
-        element_counter = self.element_map[self.points][ufl_function.element()]
+        element_counter = self.element_map[1 if avg else self.points][ufl_function.element()]
 
         # Get current cell entity, with current restriction considered
         entity = self._get_current_entity()
@@ -981,7 +1019,7 @@ class QuadratureTransformerBase(Transformer):
 
         # Create basis name and map to correct basis and get info.
         generate_psi_name = format["psi name"]
-        psi_name = generate_psi_name(element_counter, self.entitytype, entity, component, deriv)
+        psi_name = generate_psi_name(element_counter, self.entitytype, entity, component, deriv, avg)
         psi_name, non_zeros, zeros, ones = self.name_map[psi_name]
 
         # If all basis are zero we just return None.
