@@ -34,7 +34,7 @@ from expr import Expr
 
 class Symbol(Expr):
     __slots__ = ("v", "base_expr", "base_op", "exp", "cond")
-    def __init__(self, variable, symbol_type, base_expr=None, base_op=0, expo=None, cond=()):
+    def __init__(self, variable, symbol_type, base_expr=None, base_op=0):
         """Initialise a Symbols object, it derives from Expr and contains
         the additional variables:
 
@@ -57,8 +57,6 @@ class Symbol(Expr):
         # ops = base_expr.ops() + base_ops = 2 + 1 = 3
         self.base_expr = base_expr
         self.base_op = base_op
-        self.exp = expo
-        self.cond = cond
 
         # If type of the base_expr is lower than the given symbol_type change type.
         # TODO: Should we raise an error here? Or simply require that one
@@ -69,15 +67,9 @@ class Symbol(Expr):
         # Compute the representation now, such that we can use it directly
         # in the __eq__ and __ne__ methods (improves performance a bit, but
         # only when objects are cached).
-        if self.base_expr and self.exp is None:
+        if self.base_expr:# and self.exp is None:
             self._repr = "Symbol('%s', %s, %s, %d)" % (self.v, type_to_string[self.t],\
                          self.base_expr._repr, self.base_op)
-        elif self.base_expr:
-            self._repr = "Symbol('%s', %s, %s, %d, %s)" % (self.v, type_to_string[self.t],\
-                         self.base_expr._repr, self.base_op, self.exp)
-        elif self.cond:
-            self._repr = "Symbol('%s', %s, %s, %d, %s, %s)" % (self.v, type_to_string[self.t],\
-                          self.base_expr, self.base_op, self.exp, self.cond)
         else:
             self._repr = "Symbol('%s', %s)" % (self.v, type_to_string[self.t])
 
@@ -87,16 +79,8 @@ class Symbol(Expr):
     # Print functions.
     def __str__(self):
         "Simple string representation which will appear in the generated code."
-        if self.base_expr is None:
-            if self.cond == ():
-                return self.v
-            else:
-                if len(self.cond) == 2:
-                    return self.cond[1](str(self.cond[0]))
-                return format["grouping"]("".join([str(c) for c in self.cond]))
-        elif self.exp is None:
-            return self.v(str(self.base_expr))
-        return self.v(str(self.base_expr), self.exp)
+#        print "sym str: ", self.v
+        return self.v
 
     # Binary operators.
     def __add__(self, other):
@@ -232,10 +216,6 @@ class Symbol(Expr):
         # for the base (sin(2*x + 1)) --> 2 + 1.
         if self.base_expr:
             return self.base_op + self.base_expr.ops()
-        elif self.cond:
-            if len(self.cond) == 2:
-                return self.base_op + self.cond[0].ops() + 1
-            return self.base_op + self.cond[0].ops() + self.cond[2].ops() + 1
         return self.base_op
 
 from floatvalue import FloatValue
