@@ -9,11 +9,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Garth N. Wells, 2006
 # Modified by Marie E. Rognes, 2007
@@ -23,6 +23,7 @@
 # Last changed: 2009-12-21
 
 # Python modules.
+import functools
 import numpy
 
 # FFC modules.
@@ -33,7 +34,7 @@ def build_indices(dims):
     "Create a list of all index combinations."
     if not dims: return [[]]
     ranges = listcopy(dims)
-    return reduce(outer_join, ranges, [[]])
+    return functools.reduce(outer_join, ranges, [[]])
 
 def outer_join(a, b):
     """Let a be a list of lists and b a list. We append each element
