@@ -7,6 +7,7 @@ with stored reference values.
 This script can also be used for benchmarking tabulate_tensor for all
 form files found in the 'bench' directory. To run benchmarks, use the
 option --bench.
+
 """
 
 # Copyright (C) 2010-2013 Anders Logg, Kristian B. Oelgaard and Marie E. Rognes
@@ -27,9 +28,11 @@ option --bench.
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Martin Alnaes, 2013
+# Modified by Johannes Ring, 2013
+# Modified by Kristian B. Oelgaard, 2013
+# Modified by Garth N. Wells, 2014
 #
 # First added:  2010-01-21
-# Last changed: 2013-02-14
 
 # FIXME: Need to add many more test cases. Quite a few DOLFIN forms
 # failed after the FFC tests passed.
@@ -42,6 +45,7 @@ from instant.output import get_status_output
 import time
 
 # Parameters
+debug = False # TODO: Can make this a cmdline argument, and start crashing programs in debugger automatically?
 output_tolerance = 1.e-6
 demo_directory = "../../../../demo"
 bench_directory = "../../../../bench"
@@ -67,6 +71,9 @@ ext_uflacs = [\
 _command_timings = []
 def run_command(command):
     "Run command and collect errors in log file."
+    # Debugging:
+    #print "IN DIRECTORY:", os.path.abspath(os.curdir)
+    #print "RUNNING COMMAND:", command
     t1 = time.time()
     (status, output) = get_status_output(command)
     t2 = time.time()
@@ -138,7 +145,8 @@ def generate_code(args, only_forms):
 
     begin("Generating code (%d form files found)" % len(form_files))
 
-    # TODO: Parse additional options from .ufl file? I.e. grep for some sort of tag like '#ffc: <flags>'.
+    # TODO: Parse additional options from .ufl file? I.e. grep for
+    # some sort of tag like '#ffc: <flags>'.
     special = {
         "AdaptivePoisson.ufl": "-e",
         }
@@ -227,7 +235,7 @@ def build_programs(bench, permissive):
         if os.path.isfile(os.path.join(boost_dir, inc_dir, "boost", "version.hpp")):
             boost_inc_dir = os.path.join(boost_dir, inc_dir)
             break
-    for lib_dir in ["", "lib"]:
+    for lib_dir in ["", "lib", "lib/x86_64-linux-gnu"]:
         for ext in [".so", "-mt.so", ".dylib", "-mt.dylib"]:
             _lib = os.path.join(boost_dir, lib_dir, "lib" + boost_math_tr1_lib + ext)
             if os.path.isfile(_lib):
@@ -249,11 +257,16 @@ set the environment variable BOOST_DIR.
     # Set compiler options
     compiler_options = "%s -Wall" % ufc_cflags
     if not permissive:
-        compiler_options += " -Werror"
+        compiler_options += " -Werror -pedantic"
     if bench:
         info("Benchmarking activated")
         # Takes too long to build with -O2
         #compiler_options += " -O2"
+        compiler_options += " -O3"
+        #compiler_options += " -O3 -fno-math-errno -march=native"
+    if debug:
+        info("Debugging activated")
+        compiler_options += " -g -O0"
     info("Compiler options: %s" % compiler_options)
 
     # Iterate over all files
@@ -311,7 +324,7 @@ def validate_programs(reference_dir):
     "Validate generated programs against references."
 
     # Get a list of all files
-    output_files = [f for f in os.listdir(".") if f.endswith(".out")]
+    output_files = [f for f in os.listdir(".") if f.endswith(".json")]
     output_files.sort()
 
     begin("Validating generated programs (%d programs found)" % len(output_files))
@@ -396,7 +409,8 @@ def validate_programs(reference_dir):
             info_blue("Missing reference for %s" % reference_json_file)
             reference_json_output = "{}"
 
-        # Compare json with reference using recursive diff algorithm # TODO: Write to different error file?
+        # Compare json with reference using recursive diff algorithm
+        # TODO: Write to different error file?
         from recdiff import recdiff, print_recdiff, DiffEqual
         # Assuming reference is well formed
         reference_json_output = eval(reference_json_output)
@@ -415,7 +429,7 @@ def validate_programs(reference_dir):
             info_green("%s OK" % fj)
         else:
             info_red("%s differs" % fj)
-            log_error("Json output differs for %s, diff follows (reference first, generated second)"
+            log_error("Json output differs for %s, diff follows (generated first, reference second)"
                       % os.path.join(*reference_json_file.split(os.path.sep)[-3:]))
             print_recdiff(json_diff, printer=log_error)
 
@@ -428,23 +442,25 @@ def main(args):
     generate_only  = "--generate-only" in args
     fast           = "--fast" in args
     bench          = "--bench" in args
-    use_quad       = "--skip_quad" not in args
-    use_ext_quad   = "--ext_quad" in args
-    use_ext_uflacs = "--ext_uflacs" in args
+    use_quad       = "--skip-quad" not in args
+    use_ext_quad   = "--ext-quad" in args
+    use_ext_uflacs = "--ext-uflacs" in args
     permissive     = "--permissive" in args
     tolerant       = "--tolerant" in args
     print_timing   = "--print-timing" in args
+    skip_download  = "--skip-download" in args
 
     flags = (
         "--generate-only",
         "--fast",
         "--bench",
-        "--skip_quad",
-        "--ext_quad",
-        "--ext_uflacs",
+        "--skip-quad",
+        "--ext-quad",
+        "--ext-uflacs",
         "--permissive",
         "--tolerant",
         "--print-timing",
+        "--skip-download",
         )
     args = [arg for arg in args if not arg in flags]
 
@@ -452,6 +468,17 @@ def main(args):
     only_forms = set([arg for arg in args if arg.endswith(".ufl")])
     args = [arg for arg in args if arg not in only_forms]
 
+    # Download reference data
+    if skip_download:
+        info_blue("Skipping reference data download")
+    else:
+        failure, output = get_status_output("./scripts/download")
+        print output
+        if failure:
+            info_red("Download reference data failed")
+        else:
+            info_green("Download reference data ok")
+
     if tolerant:
         global output_tolerance
         output_tolerance = 1e-3
@@ -470,7 +497,7 @@ def main(args):
         test_cases += ext_quad
     if use_ext_uflacs:
         test_cases = ext_uflacs
-        #test_cases += ["-r quadrature -O"]
+        test_cases += ["-r quadrature -O"]
 
     for argument in test_cases:
 
@@ -488,9 +515,13 @@ def main(args):
         generate_code(args + [argument], only_forms)
 
         # Location of reference directories
-        reference_directory =  os.path.abspath("../../references/")
+        reference_directory =  os.path.abspath("../../ffc-reference-data/")
         code_reference_dir = os.path.join(reference_directory, sub_directory)
-        output_reference_dir = os.path.join(reference_directory, "output")
+
+        # Note: We use the r_auto references for all test cases. This
+        # ensures that we continously test that the codes generated by
+        # all different representations are equivalent.
+        output_reference_dir = os.path.join(reference_directory, "r_auto")
 
         # Validate code by comparing to code generated with this set
         # of compiler parameters
