@@ -48,6 +48,50 @@ double time()
   return ((double) (__toc_time)) / CLOCKS_PER_SEC;
 }
 
+// Function for creating "random" vertex coordinates
+std::vector<double> test_vertex_coordinates(int gdim)
+{
+  // Generate some "random" coordinates
+  std::vector<double> vertex_coordinates;
+  if (gdim == 1)
+  {
+    vertex_coordinates.resize(4);
+    vertex_coordinates[0]  = 0.903;
+    vertex_coordinates[1]  = 0.561;
+    vertex_coordinates[2]  = 0.987;
+    vertex_coordinates[3]  = 0.123;
+  }
+  else if (gdim == 2)
+  {
+    vertex_coordinates.resize(8);
+    vertex_coordinates[0]  = 0.903;
+    vertex_coordinates[1]  = 0.341;
+    vertex_coordinates[2]  = 0.561;
+    vertex_coordinates[3]  = 0.767;
+    vertex_coordinates[4]  = 0.987;
+    vertex_coordinates[5]  = 0.783;
+    vertex_coordinates[6]  = 0.123;
+    vertex_coordinates[7] = 0.561;
+  }
+  else if (gdim == 3)
+  {
+    vertex_coordinates.resize(12);
+    vertex_coordinates[0]  = 0.903;
+    vertex_coordinates[1]  = 0.341;
+    vertex_coordinates[2]  = 0.457;
+    vertex_coordinates[3]  = 0.561;
+    vertex_coordinates[4]  = 0.767;
+    vertex_coordinates[5]  = 0.833;
+    vertex_coordinates[6]  = 0.987;
+    vertex_coordinates[7]  = 0.783;
+    vertex_coordinates[8]  = 0.191;
+    vertex_coordinates[9]  = 0.123;
+    vertex_coordinates[10] = 0.561;
+    vertex_coordinates[11] = 0.667;
+  }
+  return vertex_coordinates;
+}
+
 // Class for creating "random" ufc::cell objects
 class test_cell : public ufc::cell
 {
@@ -90,71 +134,16 @@ public:
     this->orientation = 1;
 
     // Generate some "random" entity indices
-    entity_indices = new std::size_t * [4];
+    entity_indices.resize(4);
     for (std::size_t i = 0; i < 4; i++)
     {
-      entity_indices[i] = new std::size_t[6];
+      entity_indices[i].resize(6);
       for (std::size_t j = 0; j < 6; j++)
         entity_indices[i][j] = i*j + offset;
     }
-
-    // Generate some "random" coordinates (old interface)
-    double** x = new double * [4];
-    for (std::size_t i = 0; i < 4; i++)
-      x[i] = new double[3];
-    x[0][0] = 0.903; x[0][1] = 0.341; x[0][2] = 0.457;
-    x[1][0] = 0.561; x[1][1] = 0.767; x[1][2] = 0.833;
-    x[2][0] = 0.987; x[2][1] = 0.783; x[2][2] = 0.191;
-    x[3][0] = 0.123; x[3][1] = 0.561; x[3][2] = 0.667;
-    coordinates = x;
-
-    // Generate some "random" coordinates
-    vertex_coordinates.resize(12);
-    if (gdim == 1)
-    {
-      vertex_coordinates[0]  = 0.903;
-      vertex_coordinates[1]  = 0.561;
-      vertex_coordinates[2]  = 0.987;
-      vertex_coordinates[3]  = 0.123;
-    }
-    else if (gdim == 2)
-    {
-      vertex_coordinates[0]  = 0.903;
-      vertex_coordinates[1]  = 0.341;
-      vertex_coordinates[2]  = 0.561;
-      vertex_coordinates[3]  = 0.767;
-      vertex_coordinates[4]  = 0.987;
-      vertex_coordinates[5]  = 0.783;
-      vertex_coordinates[6]  = 0.123;
-      vertex_coordinates[7] = 0.561;
-    }
-    else if (gdim == 3)
-    {
-      vertex_coordinates[0]  = 0.903;
-      vertex_coordinates[1]  = 0.341;
-      vertex_coordinates[2]  = 0.457;
-      vertex_coordinates[3]  = 0.561;
-      vertex_coordinates[4]  = 0.767;
-      vertex_coordinates[5]  = 0.833;
-      vertex_coordinates[6]  = 0.987;
-      vertex_coordinates[7]  = 0.783;
-      vertex_coordinates[8]  = 0.191;
-      vertex_coordinates[9]  = 0.123;
-      vertex_coordinates[10] = 0.561;
-      vertex_coordinates[11] = 0.667;
-    }
   }
 
-  ~test_cell()
-  {
-    for (std::size_t i = 0; i < 4; i++)
-    {
-      delete [] entity_indices[i];
-      delete [] coordinates[i];
-    }
-    delete [] entity_indices;
-    delete [] coordinates;
-  }
+  ~test_cell() {}
 
 };
 
@@ -165,7 +154,8 @@ public:
 
   test_function(std::size_t value_size) : value_size(value_size) {}
 
-  void evaluate(double* values, const double* coordinates, const ufc::cell& c) const
+  void evaluate(double* values, const double* coordinates,
+                const ufc::cell& c) const
   {
     for (std::size_t i = 0; i < value_size; i++)
     {
@@ -185,39 +175,43 @@ std::string format_name(std::string name, int i=-1, int j=-1)
 {
   std::stringstream s;
   s << name;
-  if (i >= 0) s << "_" << i;
-  if (j >= 0) s << "_" << j;
+  if (i >= 0)
+    s << "_" << i;
+  if (j >= 0)
+    s << "_" << j;
   return s.str();
 }
 
 // Function for testing ufc::element objects
-void test_finite_element(ufc::finite_element& element, int id, Printer & printer)
+void test_finite_element(ufc::finite_element& element, int id, Printer& printer)
 {
   printer.begin("finite_element", id);
 
   // Prepare arguments
   test_cell c(element.cell_shape(), element.geometric_dimension());
+  const std::vector<double> vertex_coordinates
+    = test_vertex_coordinates(element.geometric_dimension());
   std::size_t value_size = 1;
   for (std::size_t i = 0; i < element.value_rank(); i++)
     value_size *= element.value_dimension(i);
   std::size_t derivative_size = 1;
   for (std::size_t i = 0; i < max_derivative; i++)
     derivative_size *= c.geometric_dimension;
-  double* values = new double[element.space_dimension()*value_size*derivative_size];
-  for (std::size_t i = 0; i < element.space_dimension()*value_size*derivative_size; i++)
+  double* values
+    = new double[element.space_dimension()*value_size*derivative_size];
+  for (std::size_t i = 0;
+       i < element.space_dimension()*value_size*derivative_size; i++)
   {
     values[i] = 0.0;
   }
   double* dof_values = new double[element.space_dimension()];
   for (std::size_t i = 0; i < element.space_dimension(); i++)
-  {
     dof_values[i] = 0.0;
-  }
+
   double* vertex_values = new double[(c.topological_dimension + 1)*value_size];
   for (std::size_t i = 0; i < (c.topological_dimension + 1)*value_size; i++)
-  {
     vertex_values[i] = 0.0;
-  }
+
   double* coordinates = new double[c.geometric_dimension];
   for (std::size_t i = 0; i < c.geometric_dimension; i++)
     coordinates[i] = 0.1*static_cast<double>(i);
@@ -242,13 +236,16 @@ void test_finite_element(ufc::finite_element& element, int id, Printer & printer
   // evaluate_basis
   for (std::size_t i = 0; i < element.space_dimension(); i++)
   {
-    element.evaluate_basis(i, values, coordinates, &c.vertex_coordinates[0], 1);
+    element.evaluate_basis(i, values, coordinates,
+                           vertex_coordinates.data(), 1);
     printer.print_array("evaluate_basis:", value_size, values, i);
   }
 
   // evaluate_basis all
-  element.evaluate_basis_all(values, coordinates, &c.vertex_coordinates[0], 1);
-  printer.print_array("evaluate_basis_all", element.space_dimension()*value_size, values);
+  element.evaluate_basis_all(values, coordinates,
+                             vertex_coordinates.data(), 1);
+  printer.print_array("evaluate_basis_all",
+                      element.space_dimension()*value_size, values);
 
   // evaluate_basis_derivatives
   for (std::size_t i = 0; i < element.space_dimension(); i++)
@@ -262,9 +259,10 @@ void test_finite_element(ufc::finite_element& element, int id, Printer & printer
                                          n,
                                          values,
                                          coordinates,
-                                         &c.vertex_coordinates[0],
+                                         vertex_coordinates.data(),
                                          1);
-      printer.print_array("evaluate_basis_derivatives", value_size*num_derivatives, values, i, n);
+      printer.print_array("evaluate_basis_derivatives",
+                          value_size*num_derivatives, values, i, n);
     }
   }
 
@@ -277,7 +275,7 @@ void test_finite_element(ufc::finite_element& element, int id, Printer & printer
     element.evaluate_basis_derivatives_all(n,
                                            values,
                                            coordinates,
-                                           &c.vertex_coordinates[0],
+                                           vertex_coordinates.data(),
                                            1);
     printer.print_array("evaluate_basis_derivatives_all",
                         element.space_dimension()*value_size*num_derivatives,
@@ -287,18 +285,20 @@ void test_finite_element(ufc::finite_element& element, int id, Printer & printer
   // evaluate_dof
   for (std::size_t i = 0; i < element.space_dimension(); i++)
   {
-    dof_values[i] = element.evaluate_dof(i, f, &c.vertex_coordinates[0], 1, c);
+    dof_values[i] = element.evaluate_dof(i, f,
+                                         vertex_coordinates.data(),
+                                         1, c);
     printer.print_scalar("evaluate_dof", dof_values[i], i);
   }
 
   // evaluate_dofs
-  element.evaluate_dofs(values, f, &c.vertex_coordinates[0], 1, c);
+  element.evaluate_dofs(values, f, vertex_coordinates.data(), 1, c);
   printer.print_array("evaluate_dofs", element.space_dimension(), values);
 
   // interpolate_vertex_values
   element.interpolate_vertex_values(vertex_values,
                                     dof_values,
-                                    &c.vertex_coordinates[0],
+                                    vertex_coordinates.data(),
                                     1,
                                     c);
   printer.print_array("interpolate_vertex_values",
@@ -326,7 +326,8 @@ void test_finite_element(ufc::finite_element& element, int id, Printer & printer
 }
 
 // Function for testing ufc::element objects
-void test_dofmap(ufc::dofmap& dofmap, ufc::shape cell_shape, int id, Printer & printer)
+void test_dofmap(ufc::dofmap& dofmap, ufc::shape cell_shape, int id,
+                 Printer& printer)
 {
   printer.begin("dofmap", id);
 
@@ -338,7 +339,9 @@ void test_dofmap(ufc::dofmap& dofmap, ufc::shape cell_shape, int id, Printer & p
   num_entities[3] = 10004;
 
   test_cell c(cell_shape, dofmap.geometric_dimension());
-  std::size_t n = dofmap.max_local_dimension();
+  const std::vector<double> vertex_coordinates
+    = test_vertex_coordinates(dofmap.geometric_dimension());
+  std::size_t n = dofmap.local_dimension();
   std::size_t* dofs = new std::size_t[n];
   for (std::size_t i = 0; i < n; i++)
     dofs[i] = 0;
@@ -353,16 +356,17 @@ void test_dofmap(ufc::dofmap& dofmap, ufc::shape cell_shape, int id, Printer & p
 
   // needs_mesh_entities
   for (std::size_t d = 0; d <= c.topological_dimension; d++)
-    printer.print_scalar("needs_mesh_entities", dofmap.needs_mesh_entities(d), d);
+  {
+    printer.print_scalar("needs_mesh_entities", dofmap.needs_mesh_entities(d),
+                         d);
+  }
 
   // global_dimension
-  printer.print_scalar("global_dimension", dofmap.global_dimension(num_entities));
+  printer.print_scalar("global_dimension",
+                       dofmap.global_dimension(num_entities));
 
   // local_dimension
-  printer.print_scalar("local_dimension", dofmap.local_dimension(c));
-
-  // max_local_dimension
-  printer.print_scalar("max_local_dimension", dofmap.max_local_dimension());
+  printer.print_scalar("local_dimension", dofmap.local_dimension());
 
   // geometric_dimension
   printer.print_scalar("geometric_dimension", dofmap.geometric_dimension());
@@ -376,13 +380,14 @@ void test_dofmap(ufc::dofmap& dofmap, ufc::shape cell_shape, int id, Printer & p
 
   // tabulate_dofs
   dofmap.tabulate_dofs(dofs, num_entities, c);
-  printer.print_array("tabulate_dofs", dofmap.local_dimension(c), dofs);
+  printer.print_array("tabulate_dofs", dofmap.local_dimension(), dofs);
 
   // tabulate_facet_dofs
   for (std::size_t facet = 0; facet < num_facets; facet++)
   {
     dofmap.tabulate_facet_dofs(dofs, facet);
-    printer.print_array("tabulate_facet_dofs", dofmap.num_facet_dofs(), dofs, facet);
+    printer.print_array("tabulate_facet_dofs", dofmap.num_facet_dofs(), dofs,
+                        facet);
   }
 
   // tabulate_entity_dofs
@@ -395,14 +400,18 @@ void test_dofmap(ufc::dofmap& dofmap, ufc::shape cell_shape, int id, Printer & p
     for (std::size_t i = 0; i < num_entities[c.topological_dimension][d]; i++)
     {
       dofmap.tabulate_entity_dofs(dofs, d, i);
-      printer.print_array("tabulate_entity_dofs", dofmap.num_entity_dofs(d), dofs, d, i);
+      printer.print_array("tabulate_entity_dofs", dofmap.num_entity_dofs(d),
+                          dofs, d, i);
     }
   }
 
   // tabulate_coordinates
-  dofmap.tabulate_coordinates(coordinates, &c.vertex_coordinates[0]);
-  for (std::size_t i = 0; i < dofmap.local_dimension(c); i++)
-    printer.print_array("tabulate_coordinates", c.geometric_dimension, coordinates[i], i);
+  dofmap.tabulate_coordinates(coordinates, vertex_coordinates.data());
+  for (std::size_t i = 0; i < dofmap.local_dimension(); i++)
+  {
+    printer.print_array("tabulate_coordinates", c.geometric_dimension,
+                        coordinates[i], i);
+  }
 
   // num_sub_dofmaps
   printer.print_scalar("num_sub_dofmaps", dofmap.num_sub_dofmaps());
@@ -438,12 +447,13 @@ void test_cell_integral(ufc::cell_integral& integral,
 
   // Prepare arguments
   test_cell c(cell_shape, gdim);
+  const std::vector<double> vertex_coordinates = test_vertex_coordinates(gdim);
   double* A = new double[tensor_size];
   for(std::size_t i = 0; i < tensor_size; i++)
     A[i] = 0.0;
 
   // Call tabulate_tensor
-  integral.tabulate_tensor(A, w, &c.vertex_coordinates[0], c.orientation);
+  integral.tabulate_tensor(A, w, vertex_coordinates.data(), c.orientation);
   printer.print_array("tabulate_tensor", tensor_size, A);
 
   // Benchmark tabulate tensor
@@ -454,13 +464,14 @@ void test_cell_integral(ufc::cell_integral& integral,
     {
       double t0 = time();
       for (std::size_t i = 0; i < num_reps; i++)
-        integral.tabulate_tensor(A, w, &c.vertex_coordinates[0], c.orientation);
+      {
+        integral.tabulate_tensor(A, w, vertex_coordinates.data(),
+                                 c.orientation);
+      }
       double dt = time() - t0;
       if (dt > minimum_timing)
       {
         dt /= static_cast<double>(num_reps);
-        //std::cout << "timing required " << num_reps << " iterations" << std::endl;
-        //std::cout << "bench cell_integral::tabulate_tensor: " << dt << std::endl;
         printer.print_scalar("cell_integral_timing_iterations", num_reps);
         printer.print_scalar("cell_integral_time", dt);
         break;
@@ -489,6 +500,7 @@ void test_exterior_facet_integral(ufc::exterior_facet_integral& integral,
 
   // Prepare arguments
   test_cell c(cell_shape, gdim);
+  const std::vector<double> vertex_coordinates = test_vertex_coordinates(gdim);
   std::size_t num_facets = c.topological_dimension + 1;
   double* A = new double[tensor_size];
 
@@ -498,7 +510,7 @@ void test_exterior_facet_integral(ufc::exterior_facet_integral& integral,
     for(std::size_t i = 0; i < tensor_size; i++)
       A[i] = 0.0;
 
-    integral.tabulate_tensor(A, w, &c.vertex_coordinates[0], facet);
+    integral.tabulate_tensor(A, w, vertex_coordinates.data(), facet);
     printer.print_array("tabulate_tensor", tensor_size, A, facet);
   }
 
@@ -510,14 +522,13 @@ void test_exterior_facet_integral(ufc::exterior_facet_integral& integral,
     {
       double t0 = time();
       for (std::size_t i = 0; i < num_reps; i++)
-        integral.tabulate_tensor(A, w, &c.vertex_coordinates[0], 0);
+        integral.tabulate_tensor(A, w, vertex_coordinates.data(), 0);
       double dt = time() - t0;
       if (dt > minimum_timing)
       {
         dt /= static_cast<double>(num_reps);
-        //std::cout << "timing required " << num_reps << " iterations" << std::endl;
-        //std::cout << "bench exterior_facet_integral::tabulate_tensor: " << dt << std::endl;
-        printer.print_scalar("exterior_facet_integral_timing_iterations", num_reps);
+        printer.print_scalar("exterior_facet_integral_timing_iterations",
+                             num_reps);
         printer.print_scalar("exterior_facet_integral_time", dt);
         break;
       }
@@ -546,6 +557,10 @@ void test_interior_facet_integral(ufc::interior_facet_integral& integral,
   // Prepare arguments
   test_cell c0(cell_shape, gdim, 0);
   test_cell c1(cell_shape, gdim, 1);
+  const std::vector<double> vertex_coordinates0
+    = test_vertex_coordinates(gdim);
+  const std::vector<double> vertex_coordinates1
+    = test_vertex_coordinates(gdim);
   std::size_t num_facets = c0.topological_dimension + 1;
   double* A = new double[macro_tensor_size];
 
@@ -559,10 +574,11 @@ void test_interior_facet_integral(ufc::interior_facet_integral& integral,
 
       integral.tabulate_tensor(A,
                                w,
-                               &c0.vertex_coordinates[0],
-                               &c1.vertex_coordinates[0],
+                               vertex_coordinates0.data(),
+                               vertex_coordinates1.data(),
                                facet0, facet1);
-      printer.print_array("tabulate_tensor", macro_tensor_size, A, facet0, facet1);
+      printer.print_array("tabulate_tensor", macro_tensor_size, A,
+                          facet0, facet1);
     }
   }
 
@@ -574,19 +590,18 @@ void test_interior_facet_integral(ufc::interior_facet_integral& integral,
     {
       double t0 = time();
       for (std::size_t i = 0; i < num_reps; i++)
-        integral.tabulate_tensor(A,
-                                 w,
-                                 &c0.vertex_coordinates[0],
-                                 &c1.vertex_coordinates[0],
-                                 0,
-                                 0);
+      {
+        integral.tabulate_tensor(A, w,
+                                 vertex_coordinates0.data(),
+                                 vertex_coordinates1.data(), 0, 0);
+      }
+
       double dt = time() - t0;
       if (dt > minimum_timing)
       {
         dt /= static_cast<double>(num_reps);
-        //std::cout << "timing required " << num_reps << " iterations" << std::endl;
-        //std::cout << "bench interior_facet_integral::tabulate_tensor: " << dt << std::endl;
-        printer.print_scalar("interior_facet_integral_timing_iterations", num_reps);
+        printer.print_scalar("interior_facet_integral_timing_iterations",
+                             num_reps);
         printer.print_scalar("interior_facet_integral_time", dt);
         break;
       }
@@ -614,6 +629,7 @@ void test_point_integral(ufc::point_integral& integral,
 
   // Prepare arguments
   test_cell c(cell_shape, gdim);
+  const std::vector<double> vertex_coordinates = test_vertex_coordinates(gdim);
   std::size_t num_vertices = c.topological_dimension + 1;
   double* A = new double[tensor_size];
 
@@ -623,7 +639,7 @@ void test_point_integral(ufc::point_integral& integral,
     for(std::size_t i = 0; i < tensor_size; i++)
       A[i] = 0.0;
 
-    integral.tabulate_tensor(A, w, &c.vertex_coordinates[0], vertex);
+    integral.tabulate_tensor(A, w, vertex_coordinates.data(), vertex);
     printer.print_array("tabulate_tensor", tensor_size, A, vertex);
   }
 
@@ -635,13 +651,11 @@ void test_point_integral(ufc::point_integral& integral,
     {
       double t0 = time();
       for (std::size_t i = 0; i < num_reps; i++)
-        integral.tabulate_tensor(A, w, &c.vertex_coordinates[0], 0);
+        integral.tabulate_tensor(A, w, vertex_coordinates.data(), 0);
       double dt = time() - t0;
       if (dt > minimum_timing)
       {
         dt /= static_cast<double>(num_reps);
-        //std::cout << "timing required " << num_reps << " iterations" << std::endl;
-        //std::cout << "bench point_integral::tabulate_tensor: " << dt << std::endl;
         printer.print_scalar("point_integral_timing_iterations", num_reps);
         printer.print_scalar("point_integral_time", dt);
         break;
@@ -679,7 +693,8 @@ void test_form(ufc::form& form, bool bench, int id, Printer & printer)
     w = new double * [form.num_coefficients()];
     for (std::size_t i = 0; i < form.num_coefficients(); i++)
     {
-      ufc::finite_element* element = form.create_finite_element(form.rank() + i);
+      ufc::finite_element* element
+        = form.create_finite_element(form.rank() + i);
       const std::size_t macro_dim = 2*element->space_dimension(); // *2 for interior facet integrals
       w[i] = new double[macro_dim];
       for (std::size_t j = 0; j < macro_dim; j++)
@@ -708,10 +723,12 @@ void test_form(ufc::form& form, bool bench, int id, Printer & printer)
   printer.print_scalar("has_cell_integrals", form.has_cell_integrals());
 
   // has_exterior_facet_integrals
-  printer.print_scalar("has_exterior_facet_integrals", form.has_exterior_facet_integrals());
+  printer.print_scalar("has_exterior_facet_integrals",
+                       form.has_exterior_facet_integrals());
 
   // has_interior_facet_integrals
-  printer.print_scalar("has_interior_facet_integrals", form.has_interior_facet_integrals());
+  printer.print_scalar("has_interior_facet_integrals",
+                       form.has_interior_facet_integrals());
 
   // has_point_integrals
   printer.print_scalar("has_point_integrals", form.has_point_integrals());
@@ -720,10 +737,12 @@ void test_form(ufc::form& form, bool bench, int id, Printer & printer)
   printer.print_scalar("num_cell_domains", form.num_cell_domains());
 
   // num_exterior_facet_domains
-  printer.print_scalar("num_exterior_facet_domains", form.num_exterior_facet_domains());
+  printer.print_scalar("num_exterior_facet_domains",
+                       form.num_exterior_facet_domains());
 
   // num_interior_facet_domains
-  printer.print_scalar("num_interior_facet_domains", form.num_interior_facet_domains());
+  printer.print_scalar("num_interior_facet_domains",
+                       form.num_interior_facet_domains());
 
   // num_point_domains
   printer.print_scalar("num_point_domains", form.num_point_domains());
@@ -764,19 +783,26 @@ void test_form(ufc::form& form, bool bench, int id, Printer & printer)
 
   // create_exterior_facet_integral
   {
-    ufc::exterior_facet_integral* integral = form.create_default_exterior_facet_integral();
+    ufc::exterior_facet_integral* integral
+      = form.create_default_exterior_facet_integral();
     printer.print_scalar("default_exterior_facet_integral", (bool)integral);
     if (integral)
+    {
       test_exterior_facet_integral(*integral, cell_shape, gdim,
                                    tensor_size, w, bench, -1, printer);
+    }
     delete integral;
   }
+
   for (std::size_t i = 0; i < form.num_exterior_facet_domains(); i++)
   {
-    ufc::exterior_facet_integral* integral = form.create_exterior_facet_integral(i);
+    ufc::exterior_facet_integral* integral
+      = form.create_exterior_facet_integral(i);
     if (integral)
+    {
       test_exterior_facet_integral(*integral, cell_shape, gdim,
                                    tensor_size, w, bench, i, printer);
+    }
     delete integral;
   }
 
@@ -791,10 +817,13 @@ void test_form(ufc::form& form, bool bench, int id, Printer & printer)
   }
   for (std::size_t i = 0; i < form.num_interior_facet_domains(); i++)
   {
-    ufc::interior_facet_integral* integral = form.create_interior_facet_integral(i);
+    ufc::interior_facet_integral* integral
+      = form.create_interior_facet_integral(i);
     if (integral)
+    {
       test_interior_facet_integral(*integral, cell_shape, gdim,
                                    macro_tensor_size, w, bench, i, printer);
+    }
     delete integral;
   }
 
@@ -803,14 +832,20 @@ void test_form(ufc::form& form, bool bench, int id, Printer & printer)
     ufc::point_integral* integral = form.create_default_point_integral();
     printer.print_scalar("default_point_integral", (bool)integral);
     if (integral)
-      test_point_integral(*integral, cell_shape, gdim, tensor_size, w, bench, -1, printer);
+    {
+      test_point_integral(*integral, cell_shape, gdim, tensor_size, w, bench,
+                          -1, printer);
+    }
     delete integral;
   }
   for (std::size_t i = 0; i < form.num_point_domains(); i++)
   {
     ufc::point_integral* integral = form.create_point_integral(i);
     if (integral)
-      test_point_integral(*integral, cell_shape, gdim, tensor_size, w, bench, i, printer);
+    {
+      test_point_integral(*integral, cell_shape, gdim, tensor_size, w, bench,
+                          i, printer);
+    }
     delete integral;
   }
 
