@@ -12,11 +12,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-07-11
 # Last changed: 2010-02-01
@@ -79,7 +79,7 @@ Fraction(
     )
   ])
 )
-        print "RED: ", expr
+#        print "RED: ", expr
         red = expr.expand().reduce_vartype(IP)
         red = expr.reduce_vartype(IP)
 #        red = expr.reduce_vartype(IP)
